/*
 * nScore
 *
 * board.c
 *
 * Copyright (C) 1996-2004 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE
#include <commctrl.h>

#include "resource.h"

#include "General.h"
#include "Memory.h"
#include "String.h"
#include "Message.h"
#include "board.h"
#include "score_input.h"

/* Define */
#define WINDOW_CLASS			TEXT("board_wnd")

#define BOX_CNT					(20 * 3 + 2)
#define INDEX_S_BULL			60
#define INDEX_D_BULL			61

/* Global Variables */
typedef struct _BOARD_INFO {
	HRGN rgn;
	TCHAR score_str[BUF_SIZE];
} BOARD_INFO;

static BOARD_INFO bi[BOX_CNT];
static HDC board_hdc;
static HBITMAP board_bmp;
static HBITMAP board_bmp_ret;
static int sel_index = -1;
static int board_size;
static BOOL op_bull;

extern HINSTANCE hInst;
extern COLOR_INFO ci;

/* Local Function Prototypes */
static HRGN create_bmp_rgn(const HBITMAP hbmp, COLORREF color, const int FrameWidth, const int FrameHeight, const BOOL auto_color);
static HBITMAP stretch_bitmap(HBITMAP hbmp, int rate, const BOOL halftone);
static BOOL init_board(const HWND hWnd);
static BOOL draw_board(const HDC hdc, const RECT *rect);
static int board_hittest(const int x, const int y);
static HWND create_tooltip(const HWND hWnd);
static LRESULT CALLBACK board_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * create_bmp_rgn - rbg}bv̓F[W쐬
 */
static HRGN create_bmp_rgn(const HBITMAP hbmp, COLORREF color, const int FrameWidth, const int FrameHeight, const BOOL auto_color)
{
#define COLOR_BITS		24	// 24bit color
	HDC hdc;
	HRGN hRgn;
	HRGN hTmpRgn;
	BITMAPINFO bi;
	BITMAP bmp;
	LPBYTE bits, p;
	DWORD LineSize;
	int x, y, st;

	// rbg}bv̏擾
	if (hbmp == NULL || GetObject(hbmp, sizeof(bmp), &bmp) == 0) {
		return NULL;
	}

	// 1C̃TCYvZ (sizeof(DWORD) ̔{ɂ)
	LineSize = (bmp.bmWidth * (COLOR_BITS / 8) + 3) & ~3;

	// wb_̏
	ZeroMemory(&bi, sizeof(BITMAPINFOHEADER));
	bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bi.bmiHeader.biWidth = bmp.bmWidth;
	bi.bmiHeader.biHeight = bmp.bmHeight;
	bi.bmiHeader.biPlanes = 1;
	bi.bmiHeader.biBitCount = COLOR_BITS;
	bi.bmiHeader.biCompression = BI_RGB;
	bi.bmiHeader.biSizeImage = LineSize * bmp.bmHeight;

	// DIB̃rbgi[郁m
	bits = LocalAlloc(LMEM_FIXED, LineSize * bmp.bmHeight);
	if (bits == NULL) {
		return NULL;
	}

	hdc = GetDC(NULL);
	// rbg擾
	if (GetDIBits(hdc, hbmp, 0, bmp.bmHeight, (LPVOID)bits, &bi, DIB_RGB_COLORS) == 0) {
		ReleaseDC(NULL, hdc);
		LocalFree(bits);
		return NULL;
	}
	ReleaseDC(NULL, hdc);

	// EBhETCỸ[W̍쐬
	hRgn = CreateRectRgn(0, 0, bmp.bmWidth + FrameWidth * 2, bmp.bmHeight + FrameHeight * 2);

	if (auto_color == TRUE) {
		color = (COLORREF)(*bits << 16 | *(bits + 1) << 8 | *(bits + 2));
	}

	// rbg}bv̓F[W珜
	for (y = 0; y < bmp.bmHeight; y++) {
		x = 0;
		p = bits + LineSize * y;
		// 1C
		while (x < bmp.bmWidth) {
			// F̎n_ƏI_擾
			for (st = x; x < bmp.bmWidth &&
				(COLORREF)(*p << 16 | *(p + 1) << 8 | *(p + 2)) != color; x++, p += (COLOR_BITS / 8));
			if (st == x) {
				// Fł͂Ȃꍇ
				x++;
				p += (COLOR_BITS / 8);
				continue;
			}
			// [W瓧F̕
			hTmpRgn = CreateRectRgn(st + FrameWidth, bmp.bmHeight - y + FrameHeight,
				x + FrameWidth, bmp.bmHeight - y - 1 + FrameHeight);
			CombineRgn(hRgn, hRgn, hTmpRgn, RGN_XOR);
			DeleteObject(hTmpRgn);
		}
	}
	LocalFree(bits);
	return hRgn;
}

/*
 * stretch_bitmap - rbg}bṽTCYύX
 */
static HBITMAP stretch_bitmap(HBITMAP hbmp, int rate, const BOOL halftone)
{
	HDC hdc, todc, fromdc;
	HBITMAP hRetBmp, hToBmp, hFromBmp;
	BITMAP bmp;
	OSVERSIONINFO osvi;
	int width, height;

	GetObject(hbmp, sizeof(bmp), &bmp);
	width = bmp.bmWidth * rate / 100;
	height = bmp.bmHeight * rate / 100;

	// ]
	hdc = GetDC(NULL);
	todc = CreateCompatibleDC(hdc);
	hRetBmp = CreateCompatibleBitmap(hdc, width, height);
	hToBmp = SelectObject(todc, hRetBmp);

	// ]
	fromdc = CreateCompatibleDC(hdc);
	hFromBmp = SelectObject(fromdc, hbmp);

	//OSo[W̃`FbN
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&osvi);
	if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT && rate < 100 && halftone == TRUE) {
		SetStretchBltMode(todc, HALFTONE);
		SetBrushOrgEx(todc, 0, 0, NULL);
	} else {
		SetStretchBltMode(todc, COLORONCOLOR);
	}
	// TCYύX
	StretchBlt(todc, 0, 0, width, height,
		fromdc, 0, 0, bmp.bmWidth, bmp.bmHeight, SRCCOPY);

	SelectObject(todc, hToBmp);
	SelectObject(fromdc, hFromBmp);
	DeleteDC(fromdc);
	DeleteDC(todc);
	ReleaseDC(NULL, hdc);
	return hRetBmp;
}

/*
 * init_board - {[h̏
 */
static BOOL init_board(const HWND hWnd)
{
	HDC hdc, mdc;
	HBITMAP hRetBmp;
	HBITMAP hbmp;
	HBITMAP stretch_bmp;
	HRGN board_rgn;
	RECT rect;
	HBRUSH hBrush;

	GetClientRect(hWnd, &rect);

	// {[h摜擾
	if (board_size == 100) {
		hbmp = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP_BOARD));
	} else {
		stretch_bmp = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP_BOARD));
		hbmp = stretch_bitmap(stretch_bmp, board_size, TRUE);
		DeleteObject(stretch_bmp);
	}

	// {[h̏
	hdc = GetDC(hWnd);
	board_hdc = CreateCompatibleDC(hdc);
	board_bmp = CreateCompatibleBitmap(hdc, rect.right, rect.bottom);
	board_bmp_ret = SelectObject(board_hdc, board_bmp);
	ReleaseDC(hWnd, hdc);

	// {[h̕`
	mdc = CreateCompatibleDC(board_hdc);
	hRetBmp = SelectObject(mdc, hbmp);
	BitBlt(board_hdc, 0, 0, rect.right, rect.bottom, mdc, 0, 0, SRCCOPY);
	SelectObject(mdc, hRetBmp);
	DeleteDC(mdc);

	// wi̓hԂ
	board_rgn = create_bmp_rgn(hbmp, 0, 0, 0, TRUE);
	hBrush = CreateSolidBrush(ci.background);
	FillRgn(board_hdc, board_rgn, hBrush);
	DeleteObject(hBrush);
	DeleteObject(board_rgn);
	DeleteObject(hbmp);
	return TRUE;
}

/*
 * init_select - Ï̏
 */
static BOOL init_select(void)
{
	HBITMAP hbmp;
	HBITMAP stretch_bmp;
	int i;

	// I[W̎擾
	for (i = 0; i < BOX_CNT; i++) {
		if (board_size == 100) {
			hbmp = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP_S01 + i));
		} else {
			stretch_bmp = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP_S01 + i));
			hbmp = stretch_bitmap(stretch_bmp, board_size, FALSE);
			DeleteObject(stretch_bmp);
		}
		bi[i].rgn = create_bmp_rgn(hbmp, RGB(0, 0, 0), 0, 0, FALSE);
		DeleteObject(hbmp);
	}

	// XRA̐ݒ
	for (i = 0; i < 20; i++) {
		wsprintf(bi[i].score_str, TEXT("%s %d"), message_get_res(IDS_STRING_SINGLE), i + 1);
	}
	for (; i < 40; i++) {
		wsprintf(bi[i].score_str, TEXT("%s %d"), message_get_res(IDS_STRING_DOUBLE), i + 1 - 20);
	}
	for (; i < 60; i++) {
		wsprintf(bi[i].score_str, TEXT("%s %d"), message_get_res(IDS_STRING_TRIPLE), i + 1 - 40);
	}
	lstrcpy(bi[i++].score_str, message_get_res(IDS_STRING_S_BULL));
	lstrcpy(bi[i++].score_str, message_get_res(IDS_STRING_D_BULL));
	return TRUE;
}

/*
 * draw_board - {[h̕`
 */
static BOOL draw_board(const HDC hdc, const RECT *rect)
{
	// {[h̕`
	BitBlt(hdc, rect->left, rect->top, rect->right, rect->bottom,
		board_hdc, rect->left, rect->top, SRCCOPY);

	if (sel_index != -1) {
		// [W̐F𔽓]
		InvertRgn(hdc, bi[sel_index].rgn);
	}
	return TRUE;
}

/*
 * board_hittest - ʒuCfbNX擾
 */
static int board_hittest(const int x, const int y)
{
	int i;

	for (i = 0; i < BOX_CNT; i++) {
		OffsetRgn(bi[i].rgn, -x, -y);
		if (PtInRegion(bi[i].rgn, 0, 0) == TRUE) {
			OffsetRgn(bi[i].rgn, x, y);
			return i;
		}
		OffsetRgn(bi[i].rgn, x, y);
	}
	return -1;
}

/*
 * create_tooltip - c[`bv̍쐬
 */
static HWND create_tooltip(const HWND hWnd)
{
	TOOLINFO ti;
	HWND hToolTip;

	hToolTip = CreateWindowEx(0, TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP, 
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
		hWnd, NULL, hInst, NULL);

	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = hWnd;
	ti.hinst = NULL;
	ti.uFlags = TTF_IDISHWND;
	ti.uId = (UINT)hWnd;
	ti.lpszText = LPSTR_TEXTCALLBACK;
	SendMessage(hToolTip, TTM_ADDTOOL, 0, (LPARAM)&ti);
	return hToolTip;
}

/*
 * board_proc - EBhEvV[W
 */
static LRESULT CALLBACK board_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hdc;
	MSG _msg;
	static HWND hToolTip;
	static int tip_index = -1;
	int i;

	switch (msg) {
	case WM_CREATE:
		// EBhE쐬
		init_board(hWnd);
		hToolTip = create_tooltip(hWnd);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;
	
	case WM_DESTROY:
		SelectObject(board_hdc, board_bmp_ret);
		DeleteObject(board_bmp);
		DeleteDC(board_hdc);
		for (i = 0; i < BOX_CNT; i++) {
			DeleteObject(bi[i].rgn);
		}
		// EBhE̔j
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SETFOCUS:
		break;

	case WM_SIZE:
		break;
	
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		// `
		draw_board(hdc, &ps.rcPaint);
		EndPaint(hWnd, &ps);
		break;

	case WM_LBUTTONDOWN:
		sel_index = board_hittest(LOWORD(lParam), HIWORD(lParam));
		InvalidateRect(hWnd, NULL, FALSE);
		UpdateWindow(hWnd);
		SetCapture(hWnd);
		break;

	case WM_MOUSEMOVE:
		i = board_hittest(LOWORD(lParam), HIWORD(lParam));
		if (tip_index != i) {
			tip_index = i;
			// c[`bv\
			SendMessage(hToolTip, TTM_POP, 0, 0);
		}
		// c[`bv\
		_msg.hwnd = hWnd;
		_msg.message = msg;
		_msg.wParam = wParam;
		_msg.lParam = lParam;
		SendMessage(hToolTip, TTM_RELAYEVENT, 0, (LPARAM)&_msg);
		break;

	case WM_LBUTTONUP:
		ReleaseCapture();
		i = board_hittest(LOWORD(lParam), HIWORD(lParam));
		if (i == sel_index && i != -1) {
			if (op_bull == TRUE && (i == INDEX_S_BULL || i == INDEX_D_BULL)) {
				SendMessage(GetParent(hWnd), WM_SET_SCORE, 0, (LPARAM)message_get_res(IDS_STRING_BULL));
			} else {
				SendMessage(GetParent(hWnd), WM_SET_SCORE, 0, (LPARAM)bi[i].score_str);
			}
//			sndPlaySound(TEXT("throw.wav"), SND_ASYNC | SND_NODEFAULT);
		}
		sel_index = -1;
		InvalidateRect(hWnd, NULL, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_NOTIFY:
		if (((NMHDR *)lParam)->code == TTN_NEEDTEXT) {
			TOOLTIPTEXT *ti = (TOOLTIPTEXT *)lParam;
			POINT pt;

			// c[`bvɕ\镶̐ݒ
			GetCursorPos(&pt);
			ScreenToClient(hWnd, &pt);
			i = board_hittest(pt.x, pt.y);
			if (i != -1) {
				if (op_bull == TRUE && (i == INDEX_S_BULL || i == INDEX_D_BULL)) {
					lstrcpy(ti->szText, message_get_res(IDS_STRING_BULL));
				} else {
					ti->lpszText = bi[i].score_str;
				}
			}
		}
		break;

	case WM_INIT_SELECT:
		init_select();
		break;

	case WM_SET_BULL_TYPE:
		op_bull = (BOOL)wParam;
		break;

	case WM_SET_BOARD_ENABLE:
		EnableWindow(hWnd, wParam);
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * board_regist - EBhENX̓o^
 */
BOOL board_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)board_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_CROSS);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * board_create - EBhE̍쐬
 */
HWND board_create(const HINSTANCE hInstance, const HWND pWnd, int id, const int size)
{
	HWND hWnd;

	board_size = size;

	// EBhE̍쐬
	hWnd = CreateWindow(WINDOW_CLASS,
		TEXT(""),
		WS_CHILD | WS_VISIBLE | WS_TABSTOP,
		0, 0, BOARD_SIZE * board_size / 100, BOARD_SIZE * board_size / 100, pWnd, (HMENU)id, hInstance, NULL);
	return hWnd;
}
/* End of source */
