/*
 * nScore
 *
 * Ini.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>

#include "General.h"
#include "Memory.h"
#include "String.h"
#include "Message.h"
#include "Profile.h"
#include "score_01.h"
#include "score_cricket.h"
#include "score_countup.h"
#include "score_cricket_countup.h"
#include "score_eagleseye.h"
#include "score_halfit.h"
#include "score_rotation.h"
#include "score_splat.h"

#include "resource.h"

/* Define */

/* Global Variables */
extern OPTION_INFO op;
extern COLOR_INFO ci;

extern ZEROONE_INFO g_zi;
extern CRICKET_INFO g_cri;
extern COUNTUP_INFO g_cui;
extern CRICKET_COUNTUP_INFO g_ccui;
extern EAGLESEYE_INFO g_eei;
extern HALFIT_INFO g_hii;
extern ROTATION_INFO g_rti;
extern SPLAT_INFO g_si;

/* Local Function Prototypes */
static COLORREF get_color(const TCHAR *buf, const COLORREF def_color);
static void put_color(const TCHAR *ini_path, const TCHAR *key, const COLORREF color);

/*
 * get_color - F擾
 */
static COLORREF get_color(const TCHAR *buf, const COLORREF def_color)
{
	if (*buf == TEXT('\0')) {
		return def_color;
	}
	return _tcstol(buf, NULL, 0);
}

/*
 * ini_get_option - IvV擾
 */
BOOL ini_get_option(const TCHAR *ini_path)
{
	RECT rect;
	TCHAR buf[BUF_SIZE];
	int i;

	profile_initialize(ini_path, TRUE);

	SystemParametersInfo(SPI_GETWORKAREA, 0, &rect, 0);

	// EBhE
	op.window_rect.left = profile_get_int(TEXT("window"), TEXT("left"), 0, ini_path);
	op.window_rect.top = profile_get_int(TEXT("window"), TEXT("top"), 0, ini_path);
	op.window_rect.right = profile_get_int(TEXT("window"), TEXT("right"), 0, ini_path);
	op.window_rect.bottom = profile_get_int(TEXT("window"), TEXT("bottom"), 0, ini_path);
	if (op.window_rect.left > rect.right || op.window_rect.top > rect.bottom) {
		op.window_rect.left = 0;
		op.window_rect.top = 0;
	}
	op.window_state = profile_get_int(TEXT("window"), TEXT("state"), SW_SHOWDEFAULT, ini_path);
	op.window_status_bar = profile_get_int(TEXT("window"), TEXT("status_bar"), 1, ini_path);

	// {[h
	op.board_size = profile_get_int(TEXT("board"), TEXT("size"), -1, ini_path);

	// F
	profile_get_string(TEXT("color"), TEXT("text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
	ci.text = get_color(buf, RGB(0, 0, 0));
	profile_get_string(TEXT("color"), TEXT("background"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
	ci.background = get_color(buf, RGB(255, 255, 255));
	profile_get_string(TEXT("color"), TEXT("focus_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
	ci.focus_text = get_color(buf, RGB(255, 255, 255));
	profile_get_string(TEXT("color"), TEXT("focus_background"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
	ci.focus_background = get_color(buf, RGB(0, 0, 128));
	profile_get_string(TEXT("color"), TEXT("winner_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
	ci.winner_text = get_color(buf, RGB(255, 255, 255));
	profile_get_string(TEXT("color"), TEXT("winner_background"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
	ci.winner_background = get_color(buf, RGB(128, 0, 0));
	profile_get_string(TEXT("color"), TEXT("close_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
	ci.close_text = get_color(buf, RGB(192, 192, 192));
	profile_get_string(TEXT("color"), TEXT("spoiler_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
	ci.spoiler_text = get_color(buf, RGB(192, 192, 192));
	profile_get_string(TEXT("color"), TEXT("bust_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
	ci.bust_text = get_color(buf, RGB(128, 128, 128));
	profile_get_string(TEXT("color"), TEXT("line"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
	ci.line = get_color(buf, RGB(192, 192, 192));

	// 01Q[
	g_zi.init_score = profile_get_int(TEXT("01game"), TEXT("init_score"), 501, ini_path);
	g_zi.player_cnt = profile_get_int(TEXT("01game"), TEXT("player_cnt"), 2, ini_path);
	g_zi.round_cnt = profile_get_int(TEXT("01game"), TEXT("round_cnt"), 15, ini_path);
	g_zi.op_bull = profile_get_int(TEXT("01game"), TEXT("op_bull"), FALSE, ini_path);
	g_zi.stats_80per = profile_get_int(TEXT("01game"), TEXT("stats_80per"), FALSE, ini_path);
	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_get_string(TEXT("01game"), buf, TEXT(""), g_zi.pi[i].name, BUF_SIZE - 1, ini_path);
		wsprintf(buf, TEXT("p%d_double_in"), i + 1);
		g_zi.pi[i].double_in = profile_get_int(TEXT("01game"), buf, FALSE, ini_path);
		wsprintf(buf, TEXT("p%d_double_out"), i + 1);
		g_zi.pi[i].double_out = profile_get_int(TEXT("01game"), buf, TRUE, ini_path);
	}

	// NPbg
	g_cri.player_cnt = profile_get_int(TEXT("cricket"), TEXT("player_cnt"), 2, ini_path);
	g_cri.round_cnt = profile_get_int(TEXT("cricket"), TEXT("round_cnt"), 20, ini_path);

	g_cri.hidden = profile_get_int(TEXT("cricket"), TEXT("hidden"), FALSE, ini_path);
	g_cri.random = profile_get_int(TEXT("cricket"), TEXT("random"), FALSE, ini_path);
	g_cri.cut_throat = profile_get_int(TEXT("cricket"), TEXT("cut_throat"), FALSE, ini_path);
	g_cri.over_kill = profile_get_int(TEXT("cricket"), TEXT("over_kill"), TRUE, ini_path);
	g_cri.stats_80per = profile_get_int(TEXT("cricket"), TEXT("stats_80per"), FALSE, ini_path);

	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_get_string(TEXT("cricket"), buf, TEXT(""), g_cri.pi[i].name, BUF_SIZE - 1, ini_path);
		wsprintf(buf, TEXT("p%d_robo"), i + 1);
		g_cri.pi[i].robo = profile_get_int(TEXT("cricket"), buf, FALSE, ini_path);
		wsprintf(buf, TEXT("p%d_robo_lv"), i + 1);
		g_cri.pi[i].robo_lv = profile_get_int(TEXT("cricket"), buf, 0, ini_path);
		wsprintf(buf, TEXT("p%d_robo_index"), i + 1);
		g_cri.pi[i].robo_index = profile_get_int(TEXT("cricket"), buf, 0, ini_path);
	}

	// JEgAbv
	g_cui.player_cnt = profile_get_int(TEXT("countup"), TEXT("player_cnt"), 1, ini_path);
	g_cui.round_cnt = COUNTUP_ROUND_MAX;
	g_cui.op_bull = profile_get_int(TEXT("countup"), TEXT("op_bull"), TRUE, ini_path);
	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_get_string(TEXT("countup"), buf, TEXT(""), g_cui.pi[i].name, BUF_SIZE - 1, ini_path);
	}

	// NPbgJEgAbv
	g_ccui.player_cnt = profile_get_int(TEXT("cricket_countup"), TEXT("player_cnt"), 1, ini_path);
	g_ccui.round_cnt = CRICKET_COUNTUP_ROUND_MAX;
	g_ccui.op_bull = profile_get_int(TEXT("cricket_countup"), TEXT("op_bull"), FALSE, ini_path);
	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_get_string(TEXT("cricket_countup"), buf, TEXT(""), g_ccui.pi[i].name, BUF_SIZE - 1, ini_path);
	}

	// C[OYAC
	g_eei.player_cnt = profile_get_int(TEXT("eagleseye"), TEXT("player_cnt"), 1, ini_path);
	g_eei.round_cnt = EAGLESEYE_ROUND_MAX;
	g_eei.op_bull = profile_get_int(TEXT("eagleseye"), TEXT("op_bull"), FALSE, ini_path);
	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_get_string(TEXT("eagleseye"), buf, TEXT(""), g_eei.pi[i].name, BUF_SIZE - 1, ini_path);
	}

	// n[tCbg
	g_hii.player_cnt = profile_get_int(TEXT("halfit"), TEXT("player_cnt"), 1, ini_path);
	g_hii.round_cnt = HALFIT_ROUND_MAX;
	g_hii.op_bull = profile_get_int(TEXT("halfit"), TEXT("op_bull"), FALSE, ini_path);
	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_get_string(TEXT("halfit"), buf, TEXT(""), g_hii.pi[i].name, BUF_SIZE - 1, ini_path);
	}

	// [e[V
	g_rti.player_cnt = profile_get_int(TEXT("rotation"), TEXT("player_cnt"), 1, ini_path);
	g_rti.round_cnt = ROTATION_ROUND_MAX;
	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_get_string(TEXT("rotation"), buf, TEXT(""), g_rti.pi[i].name, BUF_SIZE - 1, ini_path);
	}

	// Xvbg
	g_si.player_cnt = profile_get_int(TEXT("splat"), TEXT("player_cnt"), 2, ini_path);
	g_si.round_cnt = profile_get_int(TEXT("splat"), TEXT("round_cnt"), 15, ini_path);
	g_si.op_bull = profile_get_int(TEXT("splat"), TEXT("op_bull"), FALSE, ini_path);
	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_get_string(TEXT("splat"), buf, TEXT(""), g_si.pi[i].name, BUF_SIZE - 1, ini_path);
		wsprintf(buf, TEXT("p%d_double_in"), i + 1);
		g_si.pi[i].double_in = profile_get_int(TEXT("splat"), buf, FALSE, ini_path);
		wsprintf(buf, TEXT("p%d_double_out"), i + 1);
		g_si.pi[i].double_out = profile_get_int(TEXT("splat"), buf, FALSE, ini_path);
		wsprintf(buf, TEXT("p%d_master_splat"), i + 1);
		g_si.pi[i].master_splat = profile_get_int(TEXT("splat"), buf, FALSE, ini_path);
	}

	profile_free();
	return TRUE;
}

/*
 * put_color - F
 */
static void put_color(const TCHAR *ini_path, const TCHAR *key, const COLORREF color)
{
	TCHAR buf[BUF_SIZE];

	wsprintf(buf, TEXT("0x%06lX"), color);
	profile_write_string(TEXT("color"), key, buf, ini_path);
}

/*
 * ini_put_option - IvV
 */
BOOL ini_put_option(const TCHAR *ini_path)
{
	profile_initialize(ini_path, TRUE);

	// EBhE
	profile_write_int(TEXT("window"), TEXT("left"), op.window_rect.left, ini_path);
	profile_write_int(TEXT("window"), TEXT("top"), op.window_rect.top, ini_path);
	profile_write_int(TEXT("window"), TEXT("right"), op.window_rect.right, ini_path);
	profile_write_int(TEXT("window"), TEXT("bottom"), op.window_rect.bottom, ini_path);
	profile_write_int(TEXT("window"), TEXT("state"), op.window_state, ini_path);
	profile_write_int(TEXT("window"), TEXT("status_bar"), op.window_status_bar, ini_path);

	// {[h
	profile_write_int(TEXT("board"), TEXT("size"), op.board_size, ini_path);

	// F
	put_color(ini_path, TEXT("text"), ci.text);
	put_color(ini_path, TEXT("background"), ci.background);
	put_color(ini_path, TEXT("focus_text"), ci.focus_text);
	put_color(ini_path, TEXT("focus_background"), ci.focus_background);
	put_color(ini_path, TEXT("winner_text"), ci.winner_text);
	put_color(ini_path, TEXT("winner_background"), ci.winner_background);
	put_color(ini_path, TEXT("close_text"), ci.close_text);
	put_color(ini_path, TEXT("spoiler_text"), ci.spoiler_text);
	put_color(ini_path, TEXT("bust_text"), ci.bust_text);
	put_color(ini_path, TEXT("line"), ci.line);

	profile_flush(ini_path);
	profile_free();
	return TRUE;
}

/*
 * ini_put_01game_option - 01Q[̃IvV
 */
BOOL ini_put_01game_option(const TCHAR *ini_path, const ZEROONE_INFO *zi)
{
	TCHAR buf[BUF_SIZE];
	int i;

	profile_initialize(ini_path, TRUE);

	profile_write_int(TEXT("01game"), TEXT("init_score"), zi->init_score, ini_path);
	profile_write_int(TEXT("01game"), TEXT("player_cnt"), zi->player_cnt, ini_path);
	profile_write_int(TEXT("01game"), TEXT("round_cnt"), zi->round_cnt, ini_path);
	profile_write_int(TEXT("01game"), TEXT("op_bull"), zi->op_bull, ini_path);
	profile_write_int(TEXT("01game"), TEXT("stats_80per"), zi->stats_80per, ini_path);

	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_write_string(TEXT("01game"), buf, zi->pi[i].name, ini_path);
		wsprintf(buf, TEXT("p%d_double_in"), i + 1);
		profile_write_int(TEXT("01game"), buf, zi->pi[i].double_in, ini_path);
		wsprintf(buf, TEXT("p%d_double_out"), i + 1);
		profile_write_int(TEXT("01game"), buf, zi->pi[i].double_out, ini_path);
	}
	profile_flush(ini_path);
	profile_free();
	return TRUE;
}

/*
 * ini_put_cricket_option - NPbg̃IvV
 */
BOOL ini_put_cricket_option(const TCHAR *ini_path, const CRICKET_INFO *cri)
{
	TCHAR buf[BUF_SIZE];
	int i;

	profile_initialize(ini_path, TRUE);

	profile_write_int(TEXT("cricket"), TEXT("player_cnt"), cri->player_cnt, ini_path);
	profile_write_int(TEXT("cricket"), TEXT("round_cnt"), cri->round_cnt, ini_path);

	profile_write_int(TEXT("cricket"), TEXT("hidden"), cri->hidden, ini_path);
	profile_write_int(TEXT("cricket"), TEXT("random"), cri->random, ini_path);
	profile_write_int(TEXT("cricket"), TEXT("cut_throat"), cri->cut_throat, ini_path);
	profile_write_int(TEXT("cricket"), TEXT("over_kill"), cri->over_kill, ini_path);
	profile_write_int(TEXT("cricket"), TEXT("stats_80per"), cri->stats_80per, ini_path);

	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_write_string(TEXT("cricket"), buf, cri->pi[i].name, ini_path);
		wsprintf(buf, TEXT("p%d_robo"), i + 1);
		profile_write_int(TEXT("cricket"), buf, cri->pi[i].robo, ini_path);
		wsprintf(buf, TEXT("p%d_robo_lv"), i + 1);
		profile_write_int(TEXT("cricket"), buf, cri->pi[i].robo_lv, ini_path);
		wsprintf(buf, TEXT("p%d_robo_index"), i + 1);
		profile_write_int(TEXT("cricket"), buf, cri->pi[i].robo_index, ini_path);
	}
	profile_flush(ini_path);
	profile_free();
	return TRUE;
}

/*
 * ini_put_countup_option - JEgAbṽIvV
 */
BOOL ini_put_countup_option(const TCHAR *ini_path, const COUNTUP_INFO *cui)
{
	TCHAR buf[BUF_SIZE];
	int i;

	profile_initialize(ini_path, TRUE);

	profile_write_int(TEXT("countup"), TEXT("player_cnt"), cui->player_cnt, ini_path);
	profile_write_int(TEXT("countup"), TEXT("op_bull"), cui->op_bull, ini_path);

	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_write_string(TEXT("countup"), buf, cui->pi[i].name, ini_path);
	}
	profile_flush(ini_path);
	profile_free();
	return TRUE;
}

/*
 * ini_put_cricket_countup_option - NPbgJEgAbṽIvV
 */
BOOL ini_put_cricket_countup_option(const TCHAR *ini_path, const CRICKET_COUNTUP_INFO *ccui)
{
	TCHAR buf[BUF_SIZE];
	int i;

	profile_initialize(ini_path, TRUE);

	profile_write_int(TEXT("cricket_countup"), TEXT("player_cnt"), ccui->player_cnt, ini_path);
	profile_write_int(TEXT("cricket_countup"), TEXT("op_bull"), ccui->op_bull, ini_path);

	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_write_string(TEXT("cricket_countup"), buf, ccui->pi[i].name, ini_path);
	}
	profile_flush(ini_path);
	profile_free();
	return TRUE;
}

/*
 * ini_put_eagleseye_option - C[OYAC̃IvV
 */
BOOL ini_put_eagleseye_option(const TCHAR *ini_path, const EAGLESEYE_INFO *eei)
{
	TCHAR buf[BUF_SIZE];
	int i;

	profile_initialize(ini_path, TRUE);

	profile_write_int(TEXT("eagleseye"), TEXT("player_cnt"), eei->player_cnt, ini_path);
	profile_write_int(TEXT("eagleseye"), TEXT("op_bull"), eei->op_bull, ini_path);

	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_write_string(TEXT("eagleseye"), buf, eei->pi[i].name, ini_path);
	}
	profile_flush(ini_path);
	profile_free();
	return TRUE;
}

/*
 * ini_put_halfit_option - n[tCbg̃IvV
 */
BOOL ini_put_halfit_option(const TCHAR *ini_path, const HALFIT_INFO *hii)
{
	TCHAR buf[BUF_SIZE];
	int i;

	profile_initialize(ini_path, TRUE);

	profile_write_int(TEXT("halfit"), TEXT("player_cnt"), hii->player_cnt, ini_path);
	profile_write_int(TEXT("halfit"), TEXT("op_bull"), hii->op_bull, ini_path);

	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_write_string(TEXT("halfit"), buf, hii->pi[i].name, ini_path);
	}
	profile_flush(ini_path);
	profile_free();
	return TRUE;
}

/*
 * ini_put_rotation_option - [e[ṼIvV
 */
BOOL ini_put_rotation_option(const TCHAR *ini_path, const ROTATION_INFO *rti)
{
	TCHAR buf[BUF_SIZE];
	int i;

	profile_initialize(ini_path, TRUE);

	profile_write_int(TEXT("rotation"), TEXT("player_cnt"), rti->player_cnt, ini_path);

	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_write_string(TEXT("rotation"), buf, rti->pi[i].name, ini_path);
	}
	profile_flush(ini_path);
	profile_free();
	return TRUE;
}

/*
 * ini_put_splat_option - Xvbg̃IvV
 */
BOOL ini_put_splat_option(const TCHAR *ini_path, const SPLAT_INFO *si)
{
	TCHAR buf[BUF_SIZE];
	int i;

	profile_initialize(ini_path, TRUE);

	profile_write_int(TEXT("splat"), TEXT("player_cnt"), si->player_cnt, ini_path);
	profile_write_int(TEXT("splat"), TEXT("round_cnt"), si->round_cnt, ini_path);
	profile_write_int(TEXT("splat"), TEXT("op_bull"), si->op_bull, ini_path);

	for (i = 0; i < PLAYER_MAX; i++) {
		wsprintf(buf, TEXT("p%d_name"), i + 1);
		profile_write_string(TEXT("splat"), buf, si->pi[i].name, ini_path);
		wsprintf(buf, TEXT("p%d_double_in"), i + 1);
		profile_write_int(TEXT("splat"), buf, si->pi[i].double_in, ini_path);
		wsprintf(buf, TEXT("p%d_double_out"), i + 1);
		profile_write_int(TEXT("splat"), buf, si->pi[i].double_out, ini_path);
		wsprintf(buf, TEXT("p%d_master_splat"), i + 1);
		profile_write_int(TEXT("splat"), buf, si->pi[i].master_splat, ini_path);
	}
	profile_flush(ini_path);
	profile_free();
	return TRUE;
}
/* End of source */
