/*
 * nScore
 *
 * score_input.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <imm.h>
#include <tchar.h>

#include "General.h"
#include "Memory.h"
#include "string.h"
#include "Message.h"
#include "font.h"
#include "game.h"
#include "score_input.h"

#include "resource.h"

/* Define */
#define ERROR_TITLE					TEXT("nScore")

#define ID_TIMER_NEXT				1
#define TIMER_INTERVAL_NEXT			1000

#define IsNumeric(c)				(c >= TEXT('0') && c <= TEXT('9'))

/* Global Variables */
WNDPROC edit_window_proc;
WNDPROC button_window_proc;

typedef struct _INPUT_INFO {
	int current_player;
	int winner_player;
	BOOL enable;

	HWND last_focus;

	HBRUSH text_brush;
	HBRUSH background_brush;
	HBRUSH focus_background_brush;
	HBRUSH winner_background_brush;

	HFONT hfont_name;
	HFONT hfont_score;
	HFONT hfont;

	WNDPROC edit_window_proc_1;
	WNDPROC edit_window_proc_2;
	WNDPROC edit_window_proc_3;
	WNDPROC button_window_proc;
} INPUT_INFO;

extern COLOR_INFO ci;
extern BYTE char_set;

/* Local Function Prototypes */
static LRESULT CALLBACK subclass_edit_proc(const HWND hWnd, const UINT msg, const WPARAM wParam, const LPARAM lParam);
static LRESULT CALLBACK subclass_button_proc(const HWND hWnd, const UINT msg, const WPARAM wParam, const LPARAM lParam);
static void set_subclass(const HWND hDlg, INPUT_INFO *ii);
static void del_subclass(const HWND hDlg, INPUT_INFO *ii);
static BOOL CALLBACK score_input_proc(const HWND hDlg, const UINT uMsg, WPARAM wParam, LPARAM lParam);

/*
 * get_status_string - Xe[^X̎擾
 */
BOOL get_status_string(const TCHAR throw_str[THROW_MAX][BUF_SIZE], const int round_score, TCHAR *ret)
{
	int i;

	// 3 IN A BLACK
	for (i = 0; i < THROW_MAX; i++) {
		if (lstrcmpi(throw_str[i], message_get_res(IDS_STRING_D_BULL)) != 0) {
			break;
		}
	}
	if (i >= THROW_MAX) {
		lstrcpy(ret, message_get_res(IDS_STRING_3_IN_A_BLACK));
		return TRUE;
	}

	// HAT TRICK
	for (i = 0; i < THROW_MAX; i++) {
		if (lstrcmpi(throw_str[i], message_get_res(IDS_STRING_S_BULL)) != 0 &&
			lstrcmpi(throw_str[i], message_get_res(IDS_STRING_D_BULL)) != 0) {
			break;
		}
	}
	if (i >= THROW_MAX) {
		lstrcpy(ret, message_get_res(IDS_STRING_HAT_TRICK));
		return TRUE;
	}
	
	// TON 80
	if (round_score == 180) {
		lstrcpy(ret, message_get_res(IDS_STRING_TON_80));
		return TRUE;
	}

	// 3 IN A BED
	if (*throw_str[0] == CHAR_DOUBLE || *throw_str[0] == CHAR_TRIPLE) {
		for (i = 1; i < THROW_MAX; i++) {
			if (lstrcmpi(throw_str[0], throw_str[i]) != 0) {
				break;
			}
		}
		if (i >= THROW_MAX) {
			lstrcpy(ret, message_get_res(IDS_STRING_3_IN_A_BED));
			return TRUE;
		}
	}

	// LOW TON
	if (round_score >= 101 && round_score <= 150) {
		lstrcpy(ret, message_get_res(IDS_STRING_LOW_TON));
		return TRUE;
	}
	// HIGH TON
	if (round_score >= 151 && round_score <= 180) {
		lstrcpy(ret, message_get_res(IDS_STRING_HIGH_TON));
		return TRUE;
	}
	return FALSE;
}

/*
 * score_convert - XRAϊ
 */
int score_convert(TCHAR *buf)
{
	int ret;

	trim(buf);
	if (lstrcmpi(buf, message_get_res(IDS_STRING_BULL)) == 0 || lstrcmpi(buf, message_get_res(IDS_STRING_BULL_SHORT)) == 0) {
		return 50;
	}
	if (lstrcmpi(buf, message_get_res(IDS_STRING_S_BULL)) == 0 || lstrcmpi(buf, message_get_res(IDS_STRING_S_BULL_SHORT)) == 0) {
		return 25;
	}
	if (lstrcmpi(buf, message_get_res(IDS_STRING_D_BULL)) == 0 || lstrcmpi(buf, message_get_res(IDS_STRING_D_BULL_SHORT)) == 0) {
		return 50;
	}
	if (*buf == CHAR_TRIPLE || *buf == CHAR_TRIPLE_LOWER) {
		// gv
		for (buf++; *buf != TEXT('\0') && !IsNumeric(*buf); buf++);
		ret = _ttoi(buf) * 3;
	} else if (*buf == CHAR_DOUBLE || *buf == CHAR_DOUBLE_LOWER) {
		// _u
		for (buf++; *buf != TEXT('\0') && !IsNumeric(*buf); buf++);
		ret = _ttoi(buf) * 2;
	} else {
		// VO
		for (; *buf != TEXT('\0') && !IsNumeric(*buf); buf++);
		ret = _ttoi(buf);
	}
	if (ret < 0) {
		ret = 0;
	}
	return ret;
}

/*
 * score_convert_str - XRAϊ
 */
void score_convert_str(TCHAR *buf)
{
	TCHAR *p = buf;

	trim(p);
	if (lstrcmpi(p, message_get_res(IDS_STRING_BULL)) == 0 || lstrcmpi(p, message_get_res(IDS_STRING_BULL_SHORT)) == 0) {
		lstrcpy(buf, message_get_res(IDS_STRING_D_BULL));
		return;
	}
	if (lstrcmpi(p, message_get_res(IDS_STRING_S_BULL)) == 0 || lstrcmpi(buf, message_get_res(IDS_STRING_S_BULL_SHORT)) == 0) {
		lstrcpy(buf, message_get_res(IDS_STRING_S_BULL));
		return;
	}
	if (lstrcmpi(p, message_get_res(IDS_STRING_D_BULL)) == 0 || lstrcmpi(buf, message_get_res(IDS_STRING_D_BULL_SHORT)) == 0) {
		lstrcpy(buf, message_get_res(IDS_STRING_D_BULL));
		return;
	}
	if (*p == CHAR_TRIPLE || *p == CHAR_TRIPLE_LOWER) {
		// gv
		for (p++; *p != TEXT('\0') && !IsNumeric(*p); p++);
		if (*p != TEXT('\0')) {
			_itot(_ttoi(p), buf + 1, 10);
			*buf = CHAR_TRIPLE;
		} else {
			*buf = TEXT('\0');
		}
	} else if (*p == CHAR_DOUBLE || *p == CHAR_DOUBLE_LOWER) {
		// _u
		for (p++; *p != TEXT('\0') && !IsNumeric(*p); p++);
		if (*p != TEXT('\0')) {
			_itot(_ttoi(p), buf + 1, 10);
			*buf = CHAR_DOUBLE;
		} else {
			*buf = TEXT('\0');
		}
	} else {
		// VO
		for (; *p != TEXT('\0') && !IsNumeric(*p); p++);
		if (*p != TEXT('\0')) {
			_itot(_ttoi(p), buf + 1, 10);
			*buf = CHAR_SINGLE;
		} else {
			*buf = TEXT('\0');
		}
	}
}

/*
 * subclass_edit_proc - TuNXEBhEvV[W
 */
static LRESULT CALLBACK subclass_edit_proc(const HWND hWnd, const UINT msg, const WPARAM wParam, const LPARAM lParam)
{
	DWORD st, en;

	switch(msg)
	{
	case WM_KEYDOWN:
		switch (wParam) {
		case VK_LEFT:
			if (GetKeyState(VK_SHIFT) < 0) {
				break;
			}
			SendMessage(hWnd, EM_GETSEL, (WPARAM)&st, (LPARAM)&en);
			if (st == en && st == 0) {
				SendMessage(GetParent(hWnd), WM_COMMAND, ID_ACCEL_STAB, 0);
				return 0;
			}
			if (st != en) {
				SendMessage(hWnd, EM_SETSEL, st + 1, st + 1);
			}
			break;

		case VK_RIGHT:
			if (GetKeyState(VK_SHIFT) < 0) {
				break;
			}
			SendMessage(hWnd, EM_GETSEL, (WPARAM)&st, (LPARAM)&en);
			if (st == en && st == (DWORD)SendMessage(hWnd, WM_GETTEXTLENGTH, 0, 0)) {
				SendMessage(GetParent(hWnd), WM_COMMAND, ID_ACCEL_TAB, 0);
				return 0;
			}
			if (st != en) {
				SendMessage(hWnd, EM_SETSEL, en - 1, en - 1);
			}
			break;
		}
		break;
	}
	return CallWindowProc(edit_window_proc, hWnd, msg, wParam, lParam);
}

/*
 * subclass_button_proc - TuNXEBhEvV[W
 */
static LRESULT CALLBACK subclass_button_proc(const HWND hWnd, const UINT msg, const WPARAM wParam, const LPARAM lParam)
{
	switch(msg)
	{
	case WM_KEYDOWN:
		switch (wParam) {
		case VK_LEFT:
			SendMessage(GetParent(hWnd), WM_COMMAND, ID_ACCEL_STAB, 0);
			return 0;

		case VK_RIGHT:
			SendMessage(GetParent(hWnd), WM_COMMAND, ID_ACCEL_TAB, 0);
			return 0;
		}
		break;
	}
	return CallWindowProc(button_window_proc, hWnd, msg, wParam, lParam);
}

/*
 * set_subclass - EBhẼTuNX
 */
static void set_subclass(const HWND hDlg, INPUT_INFO *ii)
{
	ii->edit_window_proc_1 = (WNDPROC)SetWindowLong(GetDlgItem(hDlg, IDC_EDIT_1), GWL_WNDPROC, (long)subclass_edit_proc);
	ii->edit_window_proc_2 = (WNDPROC)SetWindowLong(GetDlgItem(hDlg, IDC_EDIT_2), GWL_WNDPROC, (long)subclass_edit_proc);
	ii->edit_window_proc_3 = (WNDPROC)SetWindowLong(GetDlgItem(hDlg, IDC_EDIT_3), GWL_WNDPROC, (long)subclass_edit_proc);

	ii->button_window_proc = (WNDPROC)SetWindowLong(GetDlgItem(hDlg, IDC_BUTTON_NEXT), GWL_WNDPROC, (long)subclass_button_proc);

	edit_window_proc = ii->edit_window_proc_1;
	button_window_proc = ii->button_window_proc;
}

/*
 * del_subclass - EBhENXŴ̂ɖ߂
 */
static void del_subclass(const HWND hDlg, INPUT_INFO *ii)
{
	if (ii->edit_window_proc_1 != NULL && IsWindow(GetDlgItem(hDlg, IDC_EDIT_1)) == TRUE) {
		SetWindowLong(GetDlgItem(hDlg, IDC_EDIT_1), GWL_WNDPROC, (long)ii->edit_window_proc_1);
		ii->edit_window_proc_1 = NULL;
	}
	if (ii->edit_window_proc_2 != NULL && IsWindow(GetDlgItem(hDlg, IDC_EDIT_2)) == TRUE) {
		SetWindowLong(GetDlgItem(hDlg, IDC_EDIT_2), GWL_WNDPROC, (long)ii->edit_window_proc_2);
		ii->edit_window_proc_2 = NULL;
	}
	if (ii->edit_window_proc_3 != NULL && IsWindow(GetDlgItem(hDlg, IDC_EDIT_3)) == TRUE) {
		SetWindowLong(GetDlgItem(hDlg, IDC_EDIT_3), GWL_WNDPROC, (long)ii->edit_window_proc_3);
		ii->edit_window_proc_3 = NULL;
	}

	if (ii->button_window_proc != NULL && IsWindow(GetDlgItem(hDlg, IDC_BUTTON_NEXT)) == TRUE) {
		SetWindowLong(GetDlgItem(hDlg, IDC_BUTTON_NEXT), GWL_WNDPROC, (long)ii->button_window_proc);
		ii->button_window_proc = NULL;
	}
}

/*
 * score_input_proc - EBhEvV[W
 */
static BOOL CALLBACK score_input_proc(const HWND hDlg, const UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	INPUT_INFO *ii;
	HWND focus_wnd;
	TCHAR buf[BUF_SIZE];
	int i;

	switch (uMsg) {
	case WM_INITDIALOG:
		ii = (INPUT_INFO *)mem_calloc(sizeof(INPUT_INFO));
		if (ii == NULL) {
			EndDialog(hDlg, FALSE);
			break;
		}
		ii->winner_player = -1;
		ii->last_focus = GetDlgItem(hDlg, IDC_EDIT_1);
		ii->text_brush = CreateSolidBrush(ci.text);
		ii->background_brush = CreateSolidBrush(ci.background);
		ii->focus_background_brush = CreateSolidBrush(ci.focus_background);
		ii->winner_background_brush = CreateSolidBrush(ci.winner_background);
		ii->hfont_name = font_create(FONT_NAME, 12, char_set, 0, FALSE, FALSE);
		ii->hfont_score = font_create(FONT_NAME, 22, 0, 0, FALSE, FALSE);
		ii->hfont = font_create(FONT_NAME, 14, 0, 0, FALSE, FALSE);
		SetWindowLong(hDlg, GWL_USERDATA, (LPARAM)ii);

		// IME𖳌ɐݒ
		ImmAssociateContext(GetDlgItem(hDlg, IDC_EDIT_1), (HIMC)NULL);
		ImmAssociateContext(GetDlgItem(hDlg, IDC_EDIT_2), (HIMC)NULL);
		ImmAssociateContext(GetDlgItem(hDlg, IDC_EDIT_3), (HIMC)NULL);

		// tHgݒ
		for (i = 0; i < PLAYER_MAX; i++) {
			SendDlgItemMessage(hDlg, IDC_STATIC_P1_NAME + i, WM_SETFONT, (WPARAM)ii->hfont_name, TRUE);
			SendDlgItemMessage(hDlg, IDC_STATIC_P1 + i, WM_SETFONT, (WPARAM)ii->hfont_score, TRUE);
		}
		SendDlgItemMessage(hDlg, IDC_EDIT_1, WM_SETFONT, (WPARAM)ii->hfont, TRUE);
		SendDlgItemMessage(hDlg, IDC_EDIT_2, WM_SETFONT, (WPARAM)ii->hfont, TRUE);
		SendDlgItemMessage(hDlg, IDC_EDIT_3, WM_SETFONT, (WPARAM)ii->hfont, TRUE);

		// TuNX
		set_subclass(hDlg, ii);

		SetFocus(GetDlgItem(hDlg, IDC_EDIT_1));
		break;

	case WM_CLOSE:
		if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) != NULL) {
			// TuNX
			del_subclass(hDlg, ii);

			DeleteObject(ii->text_brush);
			DeleteObject(ii->background_brush);
			DeleteObject(ii->focus_background_brush);
			DeleteObject(ii->winner_background_brush);
			DeleteObject(ii->hfont_name);
			DeleteObject(ii->hfont_score);
			DeleteObject(ii->hfont);
			mem_free(&ii);
		}
		SetWindowLong(hDlg, GWL_USERDATA, 0);
		EndDialog(hDlg, FALSE);
		break;

	case WM_CTLCOLORDLG:
		if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) == NULL) {
			return FALSE;
		}
		return (BOOL)ii->background_brush;

	case WM_CTLCOLORSTATIC:
		// X^eBbNRg[̐Fݒ
		if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) == NULL) {
			return FALSE;
		}
		if (ii->winner_player != -1 &&
			(GetDlgItem(hDlg, IDC_STATIC_P1_NAME + ii->winner_player) == (HWND)lParam ||
			GetDlgItem(hDlg, IDC_STATIC_P1 + ii->winner_player) == (HWND)lParam)) {
			SetTextColor((HDC)wParam, ci.winner_text);
			SetBkColor((HDC)wParam, ci.winner_background);
			return (BOOL)ii->winner_background_brush;
		}
		if (ii->winner_player == -1 &&
			(GetDlgItem(hDlg, IDC_STATIC_P1_NAME + ii->current_player) == (HWND)lParam ||
			GetDlgItem(hDlg, IDC_STATIC_P1 + ii->current_player) == (HWND)lParam)) {
			SetTextColor((HDC)wParam, ci.focus_text);
			SetBkColor((HDC)wParam, ci.focus_background);
			return (BOOL)ii->focus_background_brush;
		}
		SetTextColor((HDC)wParam, ci.text);
		SetBkColor((HDC)wParam, ci.background);
		return (BOOL)ii->background_brush;

	case WM_SETFOCUS:
		if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) == NULL) {
			return FALSE;
		}
		if (ii->last_focus == NULL) {
			if (IsWindowEnabled(GetDlgItem(hDlg, IDC_EDIT_1)) == FALSE) {
				ii->last_focus = GetDlgItem(hDlg, IDC_BUTTON_NEXT);
			} else {
				ii->last_focus = GetDlgItem(hDlg, IDC_EDIT_1);
			}
		}
		SetFocus(ii->last_focus);
		break;

	case WM_LBUTTONDBLCLK:
		{
			RECT rect;
			POINT pt;

			pt.x = LOWORD(lParam);
			pt.y = HIWORD(lParam);
			ClientToScreen(hDlg, &pt);
			
			for (i = 0; i < PLAYER_MAX; i++) {
				GetWindowRect(GetDlgItem(hDlg, IDC_STATIC_P1 + i), &rect);
				if (PtInRect(&rect, pt) == FALSE) {
					GetWindowRect(GetDlgItem(hDlg, IDC_STATIC_P1_NAME + i), &rect);
					if (PtInRect(&rect, pt) == FALSE) {
						continue;
					}
				}
				GetWindowText(GetDlgItem(hDlg, IDC_STATIC_P1 + i), buf, BUF_SIZE - 1);
				if (*buf == TEXT('\0')) {
					continue;
				}
				SendMessage(GetParent(hDlg), WM_SHOW_SCORE, i, 0);
				break;
			}
		}
		break;

	case WM_TIMER:
		switch (wParam) {
		case ID_TIMER_NEXT:
			KillTimer(hDlg, ID_TIMER_NEXT);
			if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) == NULL) {
				break;
			}
			EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_NEXT), ii->enable);
			if (IsWindowEnabled(GetDlgItem(hDlg, IDC_EDIT_1)) == FALSE) {
				ii->last_focus = GetDlgItem(hDlg, IDC_BUTTON_NEXT);
				SetFocus(ii->last_focus);
			}
			break;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_EDIT_1:
		case IDC_EDIT_2:
		case IDC_EDIT_3:
			if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) == NULL) {
				break;
			}
			switch (HIWORD(wParam)) {
			case EN_SETFOCUS:
				ii->last_focus = (HWND)lParam;
				SendMessage((HWND)lParam, EM_SETSEL, 0, -1);
				break;

			case EN_KILLFOCUS:
				SendMessage(GetParent(hDlg), WM_CHANGE_FOCUS, 0, LOWORD(wParam) - IDC_EDIT_1);
				if (GetFocus() != NULL) {
					switch (GetDlgCtrlID(GetFocus())) {
					case IDC_EDIT_1:
					case IDC_EDIT_2:
					case IDC_EDIT_3:
					case IDC_BUTTON_NEXT:
						ii->last_focus = GetFocus();
						break;
					}
				}
				break;
			}
			break;

		case IDC_BUTTON_NEXT:
			if (IsWindowEnabled(GetDlgItem(hDlg, IDC_BUTTON_NEXT)) == FALSE) {
				break;
			}
			if (SendMessage(hDlg, WM_GET_SCORE, 0, 0) > 180) {
				MessageBox(hDlg, message_get_res(IDS_STRING_INPUT_ERROR), ERROR_TITLE, MB_ICONEXCLAMATION);
				SetFocus(GetDlgItem(hDlg, IDC_EDIT_1));
				break;
			}
			if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) != NULL) {
				ii->enable = TRUE;
			}
			SendMessage(GetParent(hDlg), WM_NEXT_PLAYER, 0, 0);
			EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_NEXT), FALSE);
			SetTimer(hDlg, ID_TIMER_NEXT, TIMER_INTERVAL_NEXT, NULL);
			break;

		case ID_ACCEL_TAB:
			if (GetFocus() == NULL || GetFocus() == GetDlgItem(hDlg, IDC_BUTTON_NEXT)) {
				focus_wnd = GetDlgItem(hDlg, IDC_EDIT_1);
			} else {
				if (GetNextWindow(GetFocus(), GW_HWNDNEXT) == NULL) {
					focus_wnd = GetDlgItem(hDlg, IDC_EDIT_1);
				} else {
					focus_wnd = GetNextWindow(GetFocus(), GW_HWNDNEXT);
				}
			}
			SetFocus(focus_wnd);
			break;

		case ID_ACCEL_STAB:
			if (GetFocus() == NULL || GetFocus() == GetDlgItem(hDlg, IDC_EDIT_1)) {
				focus_wnd = GetDlgItem(hDlg, IDC_BUTTON_NEXT);
			} else {
				if (GetNextWindow(GetFocus(), GW_HWNDPREV) == NULL) {
					focus_wnd = GetDlgItem(hDlg, IDC_BUTTON_NEXT);
				} else {
					focus_wnd = GetNextWindow(GetFocus(), GW_HWNDPREV);
				}
			}
			SetFocus(focus_wnd);
			break;

		case ID_ACCEL_RETURN:
			if (GetFocus() == GetDlgItem(hDlg, IDC_BUTTON_NEXT)) {
				SendMessage(hDlg, WM_COMMAND, IDC_BUTTON_NEXT, 0);
			} else {
				SendMessage(hDlg, WM_COMMAND, ID_ACCEL_TAB, 0);
			}
			break;
		}
		break;

	case WM_INIT_SCORE:
		if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) == NULL) {
			break;
		}
		// XRȀ
		SetDlgItemText(hDlg, IDC_EDIT_1, TEXT(""));
		SetDlgItemText(hDlg, IDC_EDIT_2, TEXT(""));
		SetDlgItemText(hDlg, IDC_EDIT_3, TEXT(""));
		if (IsWindowEnabled(GetDlgItem(hDlg, IDC_EDIT_1)) == FALSE) {
			ii->last_focus = GetDlgItem(hDlg, IDC_BUTTON_NEXT);
			SetFocus(ii->last_focus);
		} else {
			SetFocus(GetDlgItem(hDlg, IDC_EDIT_1));
		}
		break;

	case WM_GET_SCORE:
		switch (wParam) {
		case 0:
		default:
			// vXRA̎擾
			i = 0;
			GetDlgItemText(hDlg, IDC_EDIT_1, buf, BUF_SIZE - 1);
			i += score_convert(buf);
			GetDlgItemText(hDlg, IDC_EDIT_2, buf, BUF_SIZE - 1);
			i += score_convert(buf);
			GetDlgItemText(hDlg, IDC_EDIT_3, buf, BUF_SIZE - 1);
			i += score_convert(buf);
			SetWindowLong(hDlg, DWL_MSGRESULT, i);
			break;

		case 1:
		case 2:
		case 3:
			// XRA̎擾
			GetDlgItemText(hDlg, IDC_EDIT_1 + wParam - 1, buf, BUF_SIZE - 1);
			SetWindowLong(hDlg, DWL_MSGRESULT, score_convert(buf));
			break;
		}
		break;

	case WM_GET_SCORE_STR:
		*buf = TEXT('\0');
		switch (wParam) {
		case 1:
		case 2:
		case 3:
			// XRA̎擾
			GetDlgItemText(hDlg, IDC_EDIT_1 + wParam - 1, buf, BUF_SIZE - 1);
			score_convert_str(buf);
			break;
		}
		if (lParam != 0) {
			lstrcpy((TCHAR *)lParam, buf);
		}
		SetWindowLong(hDlg, DWL_MSGRESULT, lstrlen(buf));
		break;

	case WM_GET_SCORE_FULL_STR:
		*buf = TEXT('\0');
		switch (wParam) {
		case 1:
		case 2:
		case 3:
			// XRA̎擾
			GetDlgItemText(hDlg, IDC_EDIT_1 + wParam - 1, buf, BUF_SIZE - 1);
			break;
		}
		if (lParam != 0) {
			lstrcpy((TCHAR *)lParam, buf);
		}
		SetWindowLong(hDlg, DWL_MSGRESULT, lstrlen(buf));
		break;

	case WM_SET_SCORE:
		// XRA̐ݒ
		if (wParam >= 1 && wParam <= 3) {
			SetWindowText(GetDlgItem(hDlg, IDC_EDIT_1 + wParam - 1), (LPCTSTR)lParam);
			break;
		}
		focus_wnd = GetFocus();
		if (focus_wnd == GetDlgItem(hDlg, IDC_BUTTON_NEXT)) {
			break;
		}
		if (focus_wnd != GetDlgItem(hDlg, IDC_EDIT_1) &&
			focus_wnd != GetDlgItem(hDlg, IDC_EDIT_2) &&
			focus_wnd != GetDlgItem(hDlg, IDC_EDIT_3)) {
			focus_wnd = GetDlgItem(hDlg, IDC_EDIT_1);
		}
		if (IsWindowEnabled(focus_wnd) == FALSE) {
			break;
		}
		SetWindowText(focus_wnd, (LPCTSTR)lParam);
		focus_wnd = GetNextWindow(focus_wnd, GW_HWNDNEXT);
		if (focus_wnd == GetDlgItem(hDlg, IDC_BUTTON_NEXT) && IsWindowEnabled(focus_wnd) == FALSE) {
			EnableWindow(focus_wnd, TRUE);
		}
		SetFocus(focus_wnd);
		break;

	case WM_SET_PLAYER_NAME:
		// vC[̖Oݒ
		if (lParam == 0) {
			SetDlgItemText(hDlg, IDC_STATIC_P1_NAME + wParam, (LPCTSTR)TEXT(""));
			break;
		}
		i = GetWindowLong(GetDlgItem(hDlg, IDC_STATIC_P1_NAME + wParam), GWL_STYLE);
		if (lstrlen((TCHAR *)lParam) > 12) {
			SetWindowLong(GetDlgItem(hDlg, IDC_STATIC_P1_NAME + wParam), GWL_STYLE, i & ~SS_CENTER);
		} else {
			SetWindowLong(GetDlgItem(hDlg, IDC_STATIC_P1_NAME + wParam), GWL_STYLE, i | SS_CENTER);
		}
		SetDlgItemText(hDlg, IDC_STATIC_P1_NAME + wParam, (LPCTSTR)lParam);
		break;

	case WM_SET_PLAYER_SCORE:
		// vC[̓_ݒ
		SetDlgItemInt(hDlg, IDC_STATIC_P1 + wParam, lParam, TRUE);
		break;

	case WM_SET_PLAYER_SCORE_STR:
		// vC[̓_ݒ
		SetDlgItemText(hDlg, IDC_STATIC_P1 + wParam, (TCHAR *)lParam);
		break;

	case WM_SET_CURRENT_PLAYER:
		// JgvC[̐ݒ
		if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) == NULL) {
			break;
		}
		ii->current_player = lParam;
		for (i = 0; i < PLAYER_MAX; i++) {
			InvalidateRect(GetDlgItem(hDlg, IDC_STATIC_P1_NAME + i), NULL, FALSE);
			UpdateWindow(GetDlgItem(hDlg, IDC_STATIC_P1_NAME + i));
			InvalidateRect(GetDlgItem(hDlg, IDC_STATIC_P1 + i), NULL, FALSE);
			UpdateWindow(GetDlgItem(hDlg, IDC_STATIC_P1 + i));
		}
		break;

	case WM_SET_WINNER_PLAYER:
		// vC[̐ݒ
		if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) == NULL) {
			break;
		}
		ii->winner_player = lParam;
		for (i = 0; i < PLAYER_MAX; i++) {
			InvalidateRect(GetDlgItem(hDlg, IDC_STATIC_P1_NAME + i), NULL, FALSE);
			UpdateWindow(GetDlgItem(hDlg, IDC_STATIC_P1_NAME + i));
			InvalidateRect(GetDlgItem(hDlg, IDC_STATIC_P1 + i), NULL, FALSE);
			UpdateWindow(GetDlgItem(hDlg, IDC_STATIC_P1 + i));
		}
		break;

	case WM_SET_SCORE_ENABLE:
		EnableWindow(GetDlgItem(hDlg, IDC_EDIT_1), wParam);
		EnableWindow(GetDlgItem(hDlg, IDC_EDIT_2), wParam);
		EnableWindow(GetDlgItem(hDlg, IDC_EDIT_3), wParam);
		if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) != NULL) {
			ii->enable = TRUE;
		}
		break;

	case WM_SET_INPUT_ENABLE:
		EnableWindow(GetDlgItem(hDlg, IDC_EDIT_1), wParam);
		EnableWindow(GetDlgItem(hDlg, IDC_EDIT_2), wParam);
		EnableWindow(GetDlgItem(hDlg, IDC_EDIT_3), wParam);
		EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_NEXT), wParam);
		if ((ii = (INPUT_INFO *)GetWindowLong(hDlg, GWL_USERDATA)) != NULL) {
			ii->enable = wParam;
		}
		break;

	case WM_SET_FOCUS_NEXT_BTN:
		if (IsWindowEnabled(GetDlgItem(hDlg, IDC_BUTTON_NEXT)) == FALSE) {
			EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_NEXT), TRUE);
		}
		SetFocus(GetDlgItem(hDlg, IDC_BUTTON_NEXT));
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

/*
 * score_input_create - EBhE쐬
 */
HWND score_input_create(const HANDLE hInst, const HWND hWnd)
{
	return CreateDialogParam(hInst, MAKEINTRESOURCE(IDD_DIALOG_SCORE), hWnd, (DLGPROC)score_input_proc, 0);
}
/* End of source */
