/*
 * nScore
 *
 * score_rotation.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>
#include <stdio.h>

#include "General.h"
#include "Memory.h"
#include "string.h"
#include "Message.h"
#include "file.h"
#include "font.h"
#include "game.h"
#include "board.h"
#include "score.h"
#include "score_rotation.h"
#include "score_rotation_list.h"
#include "score_rotation_option.h"
#include "score_input.h"

#include "resource.h"

/* Define */
#define WINDOW_CLASS				TEXT("score_rotation_wnd")
#define ERROR_TITLE					TEXT("nScore - Error")

#define WM_DRAW_SCORE				(WM_APP + 400)

/* Global Variables */
ROTATION_INFO g_rti;
static HWND input_wnd;

extern HINSTANCE hInst;
extern COLOR_INFO ci;
extern BYTE char_set;

typedef struct _BUFFER {
	HBITMAP hbmp;

	HDC draw_dc;
	HBITMAP draw_bmp;
	HBITMAP draw_ret_bmp;
	HBRUSH draw_brush;
} BUFFER;

static BUFFER bf;

/* Local Function Prototypes */
static int get_score_string_length(const ROTATION_INFO *rti);
static BOOL get_score_string(const ROTATION_INFO *rti, TCHAR *ret);
static BOOL score_save(const HWND hWnd, const ROTATION_INFO *rti);
static int get_score(const ROTATION_INFO *rti, const TCHAR *throw_str, const int score);
static BOOL round_draw(const BUFFER *draw_bf, RECT *draw_rect, const ROTATION_INFO *rti);
static BOOL score_draw(const BUFFER *draw_bf, const RECT *rect, const ROTATION_INFO *rti, const int score);
static LRESULT CALLBACK score_rotation_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * get_score_string_length - XRA̒擾
 */
static int get_score_string_length(const ROTATION_INFO *rti)
{
	TCHAR buf[BUF_SIZE];
	int i, j, t;
	int ret = 0;
	int n, m, max;

	ret += lstrlen(message_get_res(IDS_STRING_ROTATION));
	// O
	for (i = 0; i < rti->player_cnt; i++) {
		wsprintf(buf, TEXT(",%s,,,"), rti->pi[i].name);
		ret += lstrlen(buf);
	}
	ret += 2;	//CRLF
	// ^Cg
	ret += lstrlen(message_get_res(IDS_STRING_LIST_ROUND));
	for (i = 0; i < rti->player_cnt; i++) {
		for (t = 0; t < THROW_MAX; t++) {
			wsprintf(buf, TEXT(",%d"), t + 1);
			ret += lstrlen(buf);
		}
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_LIST_SCORE));
		ret += lstrlen(buf);
	}
	ret += 2;	//CRLF
	// XRA
	for (j = 0; j < rti->current_round; j++) {
		if (j == rti->current_round - 1 && rti->current_player == 0 && rti->game_flag == TRUE) {
			break;
		}
		if (j >= rti->round_cnt) {
			break;
		}
		max = rti->pi[0].ri[j].cnt;
		for (i = 1; i < rti->player_cnt; i++) {
			if (max < rti->pi[i].ri[j].cnt) {
				max = rti->pi[i].ri[j].cnt;
			}
		}
		for (n = 0; n < max; n++) {
			wsprintf(buf, TEXT("%d"), j + 1);
			ret += lstrlen(buf);
			for (i = 0; i < rti->player_cnt; i++) {
				if (n >= rti->pi[i].ri[j].cnt) {
					ret += THROW_MAX;		//,
					ret++;		//,
					continue;
				}
				// throw
				for (t = 0; t < THROW_MAX; t++) {
					ret++;		//,
					ret += lstrlen(rti->pi[i].ri[j].throw_list[n][t]);
				}
				// Score
				if (rti->pi[i].ri[j].score_list[n] < BULL_SCORE) {
					wsprintf(buf, TEXT(",%d"), rti->pi[i].ri[j].score_list[n]);
				} else {
					m = rti->pi[i].ri[j].score_list[n] - BULL_SCORE;
					if (m > BULL_COUNT) m = BULL_COUNT;
					wsprintf(buf, TEXT(",%s-%d"), message_get_res(IDS_STRING_BULL), m);
				}
				ret += lstrlen(buf);
			}
			ret += 2;	//CRLF
		}
	}
	return ret;
}

/*
 * get_score_string - XRA̍쐬
 */
static BOOL get_score_string(const ROTATION_INFO *rti, TCHAR *ret)
{
	TCHAR *p = ret;
	TCHAR buf[BUF_SIZE];
	int i, j, t;
	int n, m, max;

	lstrcpy(p, message_get_res(IDS_STRING_ROTATION));
	p += lstrlen(message_get_res(IDS_STRING_ROTATION));
	// O
	for (i = 0; i < rti->player_cnt; i++) {
		wsprintf(buf, TEXT(",%s,,,"), rti->pi[i].name);
		lstrcpy(p, buf);
		p += lstrlen(buf);
	}
	*(p++) = TEXT('\r');
	*(p++) = TEXT('\n');
	// ^Cg
	lstrcpy(p, message_get_res(IDS_STRING_LIST_ROUND));
	p += lstrlen(message_get_res(IDS_STRING_LIST_ROUND));
	for (i = 0; i < rti->player_cnt; i++) {
		for (t = 0; t < THROW_MAX; t++) {
			wsprintf(buf, TEXT(",%d"), t + 1);
			lstrcpy(p, buf);
			p += lstrlen(buf);
		}
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_LIST_SCORE));
		lstrcpy(p, buf);
		p += lstrlen(buf);
	}
	*(p++) = TEXT('\r');
	*(p++) = TEXT('\n');
	// XRA
	for (j = 0; j < rti->current_round; j++) {
		if (j == rti->current_round - 1 && rti->current_player == 0 && rti->game_flag == TRUE) {
			break;
		}
		if (j >= rti->round_cnt) {
			break;
		}
		max = rti->pi[0].ri[j].cnt;
		for (i = 1; i < rti->player_cnt; i++) {
			if (max < rti->pi[i].ri[j].cnt) {
				max = rti->pi[i].ri[j].cnt;
			}
		}
		for (n = 0; n < max; n++) {
			wsprintf(buf, TEXT("%d"), j + 1);
			lstrcpy(p, buf);
			p += lstrlen(buf);
			for (i = 0; i < rti->player_cnt; i++) {
				if (n >= rti->pi[i].ri[j].cnt) {
					for (t = 0; t < THROW_MAX; t++) {
						*(p++) = TEXT(',');
					}
					*(p++) = TEXT(',');
					continue;
				}
				// throw
				for (t = 0; t < THROW_MAX; t++) {
					*(p++) = TEXT(',');
					lstrcpy(p, rti->pi[i].ri[j].throw_list[n][t]);
					p += lstrlen(rti->pi[i].ri[j].throw_list[n][t]);
				}
				// Score
				if (rti->pi[i].ri[j].score_list[n] < BULL_SCORE) {
					wsprintf(buf, TEXT(",%d"), rti->pi[i].ri[j].score_list[n]);
				} else {
					m = rti->pi[i].ri[j].score_list[n] - BULL_SCORE;
					if (m > BULL_COUNT) m = BULL_COUNT;
					wsprintf(buf, TEXT(",%s-%d"), message_get_res(IDS_STRING_BULL), m);
				}
				lstrcpy(p, buf);
				p += lstrlen(buf);
			}
			*(p++) = TEXT('\r');
			*(p++) = TEXT('\n');
		}
	}
	*p = TEXT('\0');
	return TRUE;
}

/*
 * score_save - XRA̕ۑ
 */
static BOOL score_save(const HWND hWnd, const ROTATION_INFO *rti)
{
	TCHAR path[MAX_PATH];
	TCHAR *buf;
	TCHAR err_str[BUF_SIZE];
	TCHAR date[BUF_SIZE];
	TCHAR time[BUF_SIZE];
	DWORD size;

	// t@C̐
	if (GetDateFormat(0, 0, NULL, TEXT("yyyyMMdd"), date, BUF_SIZE - 1) == 0) {
		return FALSE;
	}
	if (GetTimeFormat(0, 0, NULL, TEXT("HHmmss"), time, BUF_SIZE - 1) == 0) {
		return FALSE;
	}
	wsprintf(path, TEXT("%s_%s_%s.csv"), message_get_res(IDS_STRING_ROTATION), date, time);

	// t@C̑I
	if (get_save_path(hWnd, message_get_res(IDS_STRING_SAVE_TITLE),
		TEXT("CSV (*.csv)\0*.csv\0All files (*.*)\0*.*\0\0"), TEXT("csv"), path) == FALSE) {
		return FALSE;
	}

	// XRA̍쐬
	size = get_score_string_length(rti);
	buf = mem_alloc(sizeof(TCHAR) * (size + 1));
	if (buf == NULL) {
		message_get_error(GetLastError(), err_str);
		MessageBox(hWnd, err_str, ERROR_TITLE, MB_ICONERROR);
		return FALSE;
	}
	get_score_string(rti, buf);

	if (file_write_buf(path, (BYTE *)buf, sizeof(TCHAR) * size, err_str) == FALSE) {
		mem_free(&buf);
		MessageBox(hWnd, err_str, ERROR_TITLE, MB_ICONERROR);
		return FALSE;
	}
	mem_free(&buf);
	return TRUE;
}

/*
 * get_score - XRA擾
 */
static int get_score(const ROTATION_INFO *rti, const TCHAR *throw_str, const int score)
{
	int cnt;
	int i;

	// ԍƃJEg擾
	if (*throw_str == TEXT('\0')) {
		return score;
	} if (lstrcmp(throw_str, message_get_res(IDS_STRING_D_BULL)) == 0 ||
		lstrcmp(throw_str, message_get_res(IDS_STRING_S_BULL)) == 0) {
		i = BULL_SCORE;
		cnt = 1;
	} else {
		switch (*throw_str) {
		case CHAR_TRIPLE:
			cnt = 3;
			break;
		case CHAR_DOUBLE:
			cnt = 2;
			break;
		case CHAR_SINGLE:
			cnt = 1;
			break;
		}
		i = _ttoi(throw_str + 1);
		if (i == 0) {
			return score;
		}
	}
	if (score >= BULL_SCORE) {
		return (score + ((i == BULL_SCORE) ? cnt : 0));
	}
	if (score != i) {
		return score;
	}
	if (score + cnt > 20) {
		return BULL_SCORE;
	}
	return (score + cnt);
}

/*
 * round_draw - Eh̕`
 */
static BOOL round_draw(const BUFFER *draw_bf, RECT *draw_rect, const ROTATION_INFO *rti)
{
	HFONT hfont, hRetFont;
	TCHAR buf[BUF_SIZE];
	int i;

	hfont = font_create(FONT_NAME, 16, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	i = (rti->current_round > rti->round_cnt) ? rti->round_cnt : rti->current_round;
	wsprintf(buf, message_get_res(IDS_STRING_DRAW_ROUND), i, rti->round_cnt);
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), draw_rect, 0);
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);
	return TRUE;
}

/*
 * score_draw - XRA̕`
 */
static BOOL score_draw(const BUFFER *draw_bf, const RECT *rect, const ROTATION_INFO *rti, const int score)
{
	TEXTMETRIC tm;
	RECT draw_rect;
	HFONT hfont, hRetFont;
	TCHAR buf[BUF_SIZE];

	draw_rect = *rect;
	FillRect(draw_bf->draw_dc, &draw_rect, draw_bf->draw_brush);
	draw_rect.left += 10;
	draw_rect.right -= 10;

	SetTextColor(draw_bf->draw_dc, ci.text);
	SetBkColor(draw_bf->draw_dc, ci.background);

	// Eh
	round_draw(draw_bf, &draw_rect, rti);

	// vC[
	draw_rect.top += 100;
	hfont = font_create(FONT_NAME, 20, char_set, 0, FALSE, FALSE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	DrawText(draw_bf->draw_dc, rti->pi[rti->current_player].name, lstrlen(rti->pi[rti->current_player].name), &draw_rect, DT_CENTER);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top += tm.tmHeight;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// XRA
	hfont = font_create(SCORE_FONT_NAME, 100, 0, FW_HEAVY, FALSE, FALSE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	if (score >= BULL_SCORE) {
		lstrcpy(buf, message_get_res(IDS_STRING_BULL));
	} else {
		_itot(score, buf, 10);
	}
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);
	return TRUE;
}

/*
 * score_rotation_proc - EBhEvV[W
 */
static LRESULT CALLBACK score_rotation_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	RECT client_rect, input_rect;
	TCHAR buf[BUF_SIZE];
	int i, j;
	BOOL change_flag;

	switch (msg) {
	case WM_CREATE:
		// ̓EBhȄ
		input_wnd = score_input_create(hInst, hWnd);
		GetClientRect(hWnd, &client_rect);
		GetClientRect(input_wnd, &input_rect);
		client_rect.right = input_rect.right;
		client_rect.bottom += input_rect.bottom;
		SetWindowPos(hWnd, 0, 0, 0, client_rect.right, client_rect.bottom, SWP_NOMOVE | SWP_NOZORDER);
		SetWindowPos(input_wnd, 0, 0, client_rect.bottom - input_rect.bottom, 0, 0, SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOZORDER);

		// `p
		hdc = GetDC(hWnd);
		client_rect.bottom -= input_rect.bottom;
		bf.draw_dc = CreateCompatibleDC(hdc);
		bf.draw_bmp = CreateCompatibleBitmap(hdc, client_rect.right, client_rect.bottom);
		bf.draw_ret_bmp = SelectObject(bf.draw_dc, bf.draw_bmp);
		ReleaseDC(hWnd, hdc);
		bf.draw_brush = CreateSolidBrush(ci.background);
		FillRect(bf.draw_dc, &client_rect, bf.draw_brush);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		// EBhE̔j
		SendMessage(input_wnd, WM_CLOSE, 0, 0);

		SelectObject(bf.draw_dc, bf.draw_ret_bmp);
		DeleteObject(bf.draw_bmp);
		DeleteDC(bf.draw_dc);
		DeleteObject(bf.draw_brush);
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SETFOCUS:
		SetFocus(input_wnd);
		break;

	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);

		BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, ps.rcPaint.bottom,
			bf.draw_dc, ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);

		EndPaint(hWnd, &ps);
		break;

	case WM_LBUTTONDBLCLK:
		break;

	case WM_GAME_OPTION:
		// rotationݒ
		return set_option_rotation(hInst, hWnd, &g_rti);

	case WM_GAME_INIT:
		// Q[̏
		g_rti.game_flag = TRUE;
		for (i = 0; i < g_rti.player_cnt; i++) {
			g_rti.pi[i].score = 1;
			SendMessage(input_wnd, WM_SET_PLAYER_NAME, i, (LPARAM)g_rti.pi[i].name);
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, i, g_rti.pi[i].score);
		}
		for (; i < PLAYER_MAX; i++) {
			SendMessage(input_wnd, WM_SET_PLAYER_NAME, i, (LPARAM)TEXT(""));
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, i, (LPARAM)TEXT(""));
		}
		g_rti.current_round = 1;
		g_rti.current_player = 0;
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_rti.pi[g_rti.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(GetParent(hWnd), WM_SET_BULL_TYPE, TRUE, 0);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_rti.current_player);
		SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, -1);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		SendMessage(hWnd, WM_DRAW_SCORE, g_rti.pi[g_rti.current_player].score, 0);
		break;

	case WM_GAME_STOP:
		break;

	case WM_CHANGE_FOCUS:
		if (g_rti.game_flag == FALSE) {
			break;
		}
		// XRA擾
		SendMessage(input_wnd, WM_GET_SCORE_STR, 1, (LPARAM)buf);
		i = get_score(&g_rti, buf, g_rti.pi[g_rti.current_player].score);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 2, (LPARAM)buf);
		i = get_score(&g_rti, buf, i);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 3, (LPARAM)buf);
		i = get_score(&g_rti, buf, i);

		if (i < BULL_SCORE) {
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_rti.current_player, i);
		} else {
			j = i - BULL_SCORE;
			if (j > BULL_COUNT) j = BULL_COUNT;
			wsprintf(buf, TEXT("%s-%d"), message_get_res(IDS_STRING_BULL), j);
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, g_rti.current_player, (LPARAM)buf);
		}
		SendMessage(hWnd, WM_DRAW_SCORE, i, 0);
		if (i >= BULL_SCORE + BULL_COUNT) {
			SendMessage(input_wnd, WM_SET_FOCUS_NEXT_BTN, 0, 0);
		}
		break;

	case WM_PREV_PLAYER:
		if (g_rti.current_player <= 0 && g_rti.current_round <= 1 && g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt <= 0) {
			break;
		}
		if (g_rti.pi[g_rti.current_player].score < BULL_SCORE) {
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_rti.current_player, g_rti.pi[g_rti.current_player].score);
		} else {
			j = g_rti.pi[g_rti.current_player].score - BULL_SCORE;
			if (j > BULL_COUNT) j = BULL_COUNT;
			wsprintf(buf, TEXT("%s-%d"), message_get_res(IDS_STRING_BULL), j);
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, g_rti.current_player, (LPARAM)buf);
		}

		if (g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt <= 0) {
			g_rti.current_player--;
			if (g_rti.current_player < 0) {
				g_rti.current_player = g_rti.player_cnt - 1;
				g_rti.current_round--;
			}
		}
		g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt--;
		if (g_rti.current_round <= 1 && g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt == 0) {
			g_rti.pi[g_rti.current_player].score = 1;
		} else if (g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt > 0) {
			g_rti.pi[g_rti.current_player].score = g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].score_list[g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt - 1];
		} else {
			g_rti.pi[g_rti.current_player].score = g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1 - 1].score_list[g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1 - 1].cnt - 1];
		}

		// 
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_rti.pi[g_rti.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_rti.current_player);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_SET_SCORE, i + 1,
				(LPARAM)g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].throw_list[g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt][i]);
		}
		SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);
		break;

	case WM_NEXT_PLAYER:
		// XRA擾
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_GET_SCORE_FULL_STR, i + 1,
				(LPARAM)g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].throw_list[g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt][i]);
		}
		// XRAݒ
		SendMessage(input_wnd, WM_GET_SCORE_STR, 1, (LPARAM)buf);
		g_rti.pi[g_rti.current_player].score = get_score(&g_rti, buf, g_rti.pi[g_rti.current_player].score);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 2, (LPARAM)buf);
		g_rti.pi[g_rti.current_player].score = get_score(&g_rti, buf, g_rti.pi[g_rti.current_player].score);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 3, (LPARAM)buf);
		i = get_score(&g_rti, buf, g_rti.pi[g_rti.current_player].score);
		change_flag = (i != g_rti.pi[g_rti.current_player].score) ? FALSE : TRUE;
		g_rti.pi[g_rti.current_player].score = i;

		g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].score_list[g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt] = g_rti.pi[g_rti.current_player].score;
		if (g_rti.pi[g_rti.current_player].score < BULL_SCORE) {
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_rti.current_player, g_rti.pi[g_rti.current_player].score);
		} else {
			j = g_rti.pi[g_rti.current_player].score - BULL_SCORE;
			if (j > BULL_COUNT) j = BULL_COUNT;
			wsprintf(buf, TEXT("%s-%d"), message_get_res(IDS_STRING_BULL), j);
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, g_rti.current_player, (LPARAM)buf);
		}
		g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt++;

		// NA
		if (g_rti.pi[g_rti.current_player].score > 52) {
			for (i = 0; i < g_rti.player_cnt; i++) {
				if (g_rti.pi[i].win == TRUE) {
					break;
				}
			}
			if (i >= g_rti.player_cnt) {
				g_rti.pi[g_rti.current_player].win = TRUE;
			}
			g_rti.pi[g_rti.current_player].clear = TRUE;
			change_flag = TRUE;

			for (i = 0; i < g_rti.player_cnt; i++) {
				if (g_rti.pi[i].clear == FALSE) {
					break;
				}
			}
			if (i >= g_rti.player_cnt) {
				//I
				g_rti.game_flag = FALSE;
				SendMessage(GetParent(hWnd), WM_GAME_END, 0, 0);
				for (i = 0; i < g_rti.player_cnt; i++) {
					if (g_rti.pi[i].win == TRUE) {
						break;
					}
				}
				SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, i);
				wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), g_rti.pi[i].name);
				SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
				MessageBox(hWnd, buf, message_get_res(IDS_STRING_TITLE_ROTATION), MB_ICONINFORMATION);
				break;
			}
		}

		if (change_flag == TRUE) {
			// vC[`FW
			do {
				g_rti.current_player++;
				if (g_rti.current_player >= g_rti.player_cnt) {
					g_rti.current_player = 0;
					g_rti.current_round++;
					if (g_rti.current_round > g_rti.round_cnt) {
						break;
					}
				}
				if (g_rti.pi[g_rti.current_player].clear == TRUE) {
					g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt = 0;
				}
			} while (g_rti.pi[g_rti.current_player].clear == TRUE);

			if (g_rti.current_round > g_rti.round_cnt) {
				// SEhI
				g_rti.game_flag = FALSE;
				SendMessage(GetParent(hWnd), WM_GAME_END, 0, 0);

				for (i = 0; i < g_rti.player_cnt; i++) {
					if (g_rti.pi[i].win == TRUE) {
						break;
					}
				}
				if (i >= g_rti.player_cnt) {
					for (j = 0, i = 1; i < g_rti.player_cnt; i++) {
						if (g_rti.pi[j].score < g_rti.pi[i].score) {
							j = i;
						}
					}
					for (i = 0; i < g_rti.player_cnt; i++) {
						if (j != i && g_rti.pi[j].score == g_rti.pi[i].score) {
							break;
						}
					}
					if (g_rti.player_cnt > 1 && g_rti.player_cnt > i) {
						// 
						lstrcpy(buf, message_get_res(IDS_STRING_TIE));
					} else {
						SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, j);
						wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), g_rti.pi[j].name);
					}
				} else {
					SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, i);
					wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), g_rti.pi[i].name);
				}
				SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
				MessageBox(hWnd, buf, message_get_res(IDS_STRING_TITLE_ROTATION), MB_ICONINFORMATION);
				break;
			}
			g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt = 0;
		}

		// 
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_rti.pi[g_rti.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_rti.current_player);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_SET_SCORE, i + 1,
				(LPARAM)g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].throw_list[g_rti.pi[g_rti.current_player].ri[g_rti.current_round - 1].cnt][i]);
		}
		SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);
		break;

	case WM_DRAW_SCORE:
		GetClientRect(hWnd, &client_rect);
		GetClientRect(input_wnd, &input_rect);
		client_rect.bottom -= input_rect.bottom;
		score_draw(&bf, &client_rect, &g_rti, wParam);
		InvalidateRect(hWnd, &client_rect, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_SHOW_SCORE:
		if (wParam != -1) {
			g_rti.list_index = wParam;
		} else {
			g_rti.list_index = (g_rti.game_flag == FALSE) ? 0 : g_rti.current_player;
		}
		show_score_rotation_list(hInst, hWnd, &g_rti);
		break;

	case WM_SAVE_SCORE:
		score_save(hWnd, &g_rti);
		break;

	case WM_SET_SCORE:
	case WM_SET_INPUT_ENABLE:
		return SendMessage(input_wnd, msg, wParam, lParam);

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_ACCEL_TAB:
		case ID_ACCEL_STAB:
		case ID_ACCEL_RETURN:
		case IDC_BUTTON_NEXT:
			SendMessage(input_wnd, WM_COMMAND, wParam, lParam);
			break;
		}
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * score_rotation_regist - EBhENX̓o^
 */
BOOL score_rotation_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = (WNDPROC)score_rotation_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * score_rotation_create - EBhE̍쐬
 */
HWND score_rotation_create(const HINSTANCE hInstance, const HWND pWnd, int id, const int height)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindow(WINDOW_CLASS,
		TEXT(""),
		WS_CHILD | WS_TABSTOP,
		0, 0, SCORE_WIDTH, height, pWnd, (HMENU)id, hInstance, NULL);
	return hWnd;
}
/* End of source */
