/*
 * nScore
 *
 * score_splat.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>
#include <stdio.h>

#include "General.h"
#include "Memory.h"
#include "string.h"
#include "Message.h"
#include "file.h"
#include "font.h"
#include "game.h"
#include "board.h"
#include "score.h"
#include "score_splat.h"
#include "score_splat_list.h"
#include "score_splat_option.h"
#include "score_input.h"

#include "resource.h"

/* Define */
#define WINDOW_CLASS				TEXT("score_splat_wnd")
#define ERROR_TITLE					TEXT("nScore - Error")

#define WM_DRAW_SCORE				(WM_APP + 400)

#define SPLAT_SCORE					333

#define SPLAT_CHAR					TEXT("X")

/* Global Variables */
SPLAT_INFO g_si;
static HWND input_wnd;

extern HINSTANCE hInst;
extern COLOR_INFO ci;
extern BYTE char_set;

typedef struct _BUFFER {
	HBITMAP hbmp;

	HDC draw_dc;
	HBITMAP draw_bmp;
	HBITMAP draw_ret_bmp;
	HBRUSH draw_brush;
} BUFFER;

static BUFFER bf;

/* Local Function Prototypes */
static int get_score_string_length(const SPLAT_INFO *si);
static BOOL get_score_string(const SPLAT_INFO *si, TCHAR *ret);
static BOOL score_save(const HWND hWnd, const SPLAT_INFO *si);
static void check_score(SPLAT_INFO *si, const int player, const int score);
static BOOL set_score(SPLAT_INFO *si, const int player);
static BOOL round_draw(const BUFFER *draw_bf, RECT *draw_rect, const SPLAT_INFO *si);
static BOOL score_draw(const BUFFER *draw_bf, const RECT *rect, const SPLAT_INFO *si, const int score, const BOOL bust);
static LRESULT CALLBACK score_splat_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * get_score_string_length - XRA̒擾
 */
static int get_score_string_length(const SPLAT_INFO *si)
{
	TCHAR buf[BUF_SIZE];
	int i, j, t;
	int ret = 0;

	ret += lstrlen(message_get_res(IDS_STRING_SPLAT));
	// O
	for (i = 0; i < si->player_cnt; i++) {
		wsprintf(buf, TEXT(",%s,,,"), si->pi[i].name);
		ret += lstrlen(buf);
	}
	ret += 2;	//CRLF
	// ^Cg
	ret += lstrlen(message_get_res(IDS_STRING_LIST_ROUND));
	for (i = 0; i < si->player_cnt; i++) {
		for (t = 0; t < THROW_MAX; t++) {
			wsprintf(buf, TEXT(",%d"), t + 1);
			ret += lstrlen(buf);
		}
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_LIST_SCORE));
		ret += lstrlen(buf);
	}
	ret += 2;	//CRLF
	// XRA
	for (j = 0; j < si->current_round; j++) {
		if (j == si->current_round - 1 && si->current_player == 0 && si->game_flag == TRUE) {
			break;
		}
		if (j >= si->round_cnt) {
			break;
		}
		wsprintf(buf, TEXT("%d"), j + 1);
		ret += lstrlen(buf);
		for (i = 0; i < si->player_cnt; i++) {
			// throw
			for (t = 0; t < THROW_MAX; t++) {
				ret++;		//,
				ret += lstrlen(si->pi[i].ri[j].throw_list[t]);
			}
			if (j == si->current_round - 1 &&
				(i > si->current_player || (si->game_flag == TRUE && i == si->current_player))) {
				ret++;		//,
				continue;
			}
			// Score
			wsprintf(buf, TEXT(",%d"), si->pi[i].ri[j].score_list);
			ret += lstrlen(buf);
		}
		ret += 2;	//CRLF
	}
	return ret;
}

/*
 * get_score_string - XRA̍쐬
 */
static BOOL get_score_string(const SPLAT_INFO *si, TCHAR *ret)
{
	TCHAR *p = ret;
	TCHAR buf[BUF_SIZE];
	int i, j, t;

	lstrcpy(p, message_get_res(IDS_STRING_SPLAT));
	p += lstrlen(message_get_res(IDS_STRING_SPLAT));
	// O
	for (i = 0; i < si->player_cnt; i++) {
		wsprintf(buf, TEXT(",%s,,,"), si->pi[i].name);
		lstrcpy(p, buf);
		p += lstrlen(buf);
	}
	*(p++) = TEXT('\r');
	*(p++) = TEXT('\n');
	// ^Cg
	lstrcpy(p, message_get_res(IDS_STRING_LIST_ROUND));
	p += lstrlen(message_get_res(IDS_STRING_LIST_ROUND));
	for (i = 0; i < si->player_cnt; i++) {
		for (t = 0; t < THROW_MAX; t++) {
			wsprintf(buf, TEXT(",%d"), t + 1);
			lstrcpy(p, buf);
			p += lstrlen(buf);
		}
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_LIST_SCORE));
		lstrcpy(p, buf);
		p += lstrlen(buf);
	}
	*(p++) = TEXT('\r');
	*(p++) = TEXT('\n');
	// XRA
	for (j = 0; j < si->current_round; j++) {
		if (j == si->current_round - 1 && si->current_player == 0 && si->game_flag == TRUE) {
			break;
		}
		if (j >= si->round_cnt) {
			break;
		}
		wsprintf(buf, TEXT("%d"), j + 1);
		lstrcpy(p, buf);
		p += lstrlen(buf);
		for (i = 0; i < si->player_cnt; i++) {
			// throw
			for (t = 0; t < THROW_MAX; t++) {
				*(p++) = TEXT(',');
				lstrcpy(p, si->pi[i].ri[j].throw_list[t]);
				p += lstrlen(si->pi[i].ri[j].throw_list[t]);
			}
			if (j == si->current_round - 1 &&
				(i > si->current_player || (si->game_flag == TRUE && i == si->current_player))) {
				*(p++) = TEXT(',');
				continue;
			}
			// Score
			wsprintf(buf, TEXT(",%d"), si->pi[i].ri[j].score_list);
			lstrcpy(p, buf);
			p += lstrlen(buf);
		}
		*(p++) = TEXT('\r');
		*(p++) = TEXT('\n');
	}
	*p = TEXT('\0');
	return TRUE;
}

/*
 * score_save - XRA̕ۑ
 */
static BOOL score_save(const HWND hWnd, const SPLAT_INFO *si)
{
	TCHAR path[MAX_PATH];
	TCHAR *buf;
	TCHAR err_str[BUF_SIZE];
	TCHAR date[BUF_SIZE];
	TCHAR time[BUF_SIZE];
	DWORD size;

	// t@C̐
	if (GetDateFormat(0, 0, NULL, TEXT("yyyyMMdd"), date, BUF_SIZE - 1) == 0) {
		return FALSE;
	}
	if (GetTimeFormat(0, 0, NULL, TEXT("HHmmss"), time, BUF_SIZE - 1) == 0) {
		return FALSE;
	}
	wsprintf(path, TEXT("%s_%s_%s.csv"), message_get_res(IDS_STRING_SPLAT), date, time);

	// t@C̑I
	if (get_save_path(hWnd, message_get_res(IDS_STRING_SAVE_TITLE),
		TEXT("CSV (*.csv)\0*.csv\0All files (*.*)\0*.*\0\0"), TEXT("csv"), path) == FALSE) {
		return FALSE;
	}

	// XRA̍쐬
	size = get_score_string_length(si);
	buf = mem_alloc(sizeof(TCHAR) * (size + 1));
	if (buf == NULL) {
		message_get_error(GetLastError(), err_str);
		MessageBox(hWnd, err_str, ERROR_TITLE, MB_ICONERROR);
		return FALSE;
	}
	get_score_string(si, buf);

	if (file_write_buf(path, (BYTE *)buf, sizeof(TCHAR) * size, err_str) == FALSE) {
		mem_free(&buf);
		MessageBox(hWnd, err_str, ERROR_TITLE, MB_ICONERROR);
		return FALSE;
	}
	mem_free(&buf);
	return TRUE;
}

/*
 * check_score - XRA_ɂȂĂȂ`FbN
 */
static void check_score(SPLAT_INFO *si, const int player, const int score)
{
	int i;

	for (i = 0; i < si->player_cnt; i++) {
		if (i != player && si->pi[i].score == score && si->pi[i].score > 0) {
			si->pi[i].half = TRUE;
			if (si->pi[i].score > 180) {
				check_score(si, i, (si->pi[i].score + 1) / 2);
			}
		}
	}
}

/*
 * set_score - XRA_ɂȂĂvC[̃XRA𔼕ɐݒ
 */
static BOOL set_score(SPLAT_INFO *si, const int player)
{
	int i;
	BOOL ret = FALSE;
	BOOL master_splat_flag;

	for (i = 0; i < si->player_cnt; i++) {
		if (i != player && si->pi[i].score == si->pi[player].score && si->pi[i].score > 0) {
			if (si->pi[i].score > 180) {
				si->pi[i].score = (si->pi[i].score + 1) / 2;
				master_splat_flag = set_score(si, i);
				if (si->pi[i].master_splat_flag == 0 && master_splat_flag == TRUE) {
					si->pi[i].master_splat_flag = si->current_round;
				}
			} else {
				si->pi[i].score = 0;
			}
			si->pi[player].ri[si->current_round - 1].splats++;
			ret = TRUE;
		}
	}
	return ret;
}

/*
 * round_draw - Eh̕`
 */
static BOOL round_draw(const BUFFER *draw_bf, RECT *draw_rect, const SPLAT_INFO *si)
{
	HFONT hfont, hRetFont;
	TCHAR buf[BUF_SIZE];
	int i;

	hfont = font_create(FONT_NAME, 16, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	i = (si->current_round > si->round_cnt) ? si->round_cnt : si->current_round;
	wsprintf(buf, message_get_res(IDS_STRING_DRAW_ROUND), i, si->round_cnt);
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), draw_rect, 0);
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);
	return TRUE;
}

/*
 * score_draw - XRA̕`
 */
static BOOL score_draw(const BUFFER *draw_bf, const RECT *rect, const SPLAT_INFO *si, const int score, const BOOL bust)
{
	TEXTMETRIC tm;
	RECT draw_rect;
	HFONT hfont, hRetFont;
	TCHAR buf[BUF_SIZE];
	int score_width;
	int j;

	draw_rect = *rect;
	FillRect(draw_bf->draw_dc, &draw_rect, draw_bf->draw_brush);
	draw_rect.left += 10;
	draw_rect.right -= 10;
	score_width = draw_rect.right - draw_rect.left;

	SetTextColor(draw_bf->draw_dc, ci.text);
	SetBkColor(draw_bf->draw_dc, ci.background);

	// Eh
	round_draw(draw_bf, &draw_rect, si);

	// NA܂ł̓_
	hfont = font_create(FONT_NAME, 16, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	DrawText(draw_bf->draw_dc, message_get_res(IDS_STRING_DRAW_REMAINDER), lstrlen(message_get_res(IDS_STRING_DRAW_REMAINDER)), &draw_rect, DT_RIGHT);
	draw_rect.top += tm.tmHeight;
	if ((si->pi[si->current_player].double_in == TRUE && si->pi[si->current_player].double_in_flag == 0 && score == 0) ||
		(si->pi[si->current_player].master_splat == TRUE && si->pi[si->current_player].master_splat_flag == 0) ||
		bust == TRUE) {
		lstrcpy(buf, TEXT("-"));
	} else {
		wsprintf(buf, TEXT("%d"), SPLAT_SCORE - score);
	}
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_RIGHT);
	draw_rect.top += tm.tmHeight;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// vC[
	hfont = font_create(FONT_NAME, 20, char_set, 0, FALSE, FALSE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	DrawText(draw_bf->draw_dc, si->pi[si->current_player].name, lstrlen(si->pi[si->current_player].name), &draw_rect, DT_CENTER);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top += tm.tmHeight + 10;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// IvV\
	hfont = font_create(FONT_NAME, 20, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	*buf = TEXT('\0');
	if (si->pi[si->current_player].double_in == TRUE && si->pi[si->current_player].double_in_flag == 0) {
		lstrcpy(buf, message_get_res(IDS_STRING_DOUBLE_IN));
	} else {
		if (si->pi[si->current_player].double_out == TRUE) {
			lstrcpy(buf, message_get_res(IDS_STRING_DOUBLE_OUT));
		}
		if (si->pi[si->current_player].master_splat == TRUE && si->pi[si->current_player].master_splat_flag == 0) {
			if (*buf != TEXT('\0')) {
				lstrcat(buf, TEXT(", "));
			}
			lstrcat(buf, message_get_res(IDS_STRING_MASTER_SPLAT));
		}
	}
	if (*buf != TEXT('\0')) {
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
	}
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top += tm.tmHeight;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// XRA
	if (bust == TRUE) {
		SetTextColor(draw_bf->draw_dc, ci.bust_text);
	}
	hfont = font_create(SCORE_FONT_NAME, 100, 0, FW_HEAVY, FALSE, FALSE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	_itot(score, buf, 10);
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top += tm.tmHeight;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// Xe[^X
	SetTextColor(draw_bf->draw_dc, ci.text);
	hfont = font_create(FONT_NAME, 16, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	for (j = 0; j < si->player_cnt; j++) {
		draw_rect.left = j * (score_width / 4) + 20;
		draw_rect.right = draw_rect.left + 100;

		if (j != si->current_player) {
			// ܂ł̓_
			if (si->pi[j].half == TRUE ||
				(si->pi[si->current_player].double_in == TRUE && si->pi[si->current_player].double_in_flag == 0 && score == 0)) {
				lstrcpy(buf, TEXT("-"));
			} else if (si->pi[j].score <= score) {
				if (SPLAT_SCORE + SPLAT_SCORE - score - si->pi[j].score < 180) {
					_itot(SPLAT_SCORE + SPLAT_SCORE - score - si->pi[j].score, buf, 10);
				} else {
					lstrcpy(buf, TEXT("-"));
				}
			} else {
				_itot(si->pi[j].score - score, buf, 10);
			}
			DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
		}
	}
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);
	return TRUE;
}

/*
 * score_splat_proc - EBhEvV[W
 */
static LRESULT CALLBACK score_splat_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	RECT client_rect, input_rect;
	TCHAR buf[BUF_SIZE];
	int i, j;
	BOOL double_in_flag;
	BOOL master_splat_flag;
	BOOL bust_flag;
	BOOL splat_flag;

	switch (msg) {
	case WM_CREATE:
		// ̓EBhȄ
		input_wnd = score_input_create(hInst, hWnd);
		GetClientRect(hWnd, &client_rect);
		GetClientRect(input_wnd, &input_rect);
		client_rect.right = input_rect.right;
		client_rect.bottom += input_rect.bottom;
		SetWindowPos(hWnd, 0, 0, 0, client_rect.right, client_rect.bottom, SWP_NOMOVE | SWP_NOZORDER);
		SetWindowPos(input_wnd, 0, 0, client_rect.bottom - input_rect.bottom, 0, 0, SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOZORDER);

		// `p
		hdc = GetDC(hWnd);
		client_rect.bottom -= input_rect.bottom;
		bf.draw_dc = CreateCompatibleDC(hdc);
		bf.draw_bmp = CreateCompatibleBitmap(hdc, client_rect.right, client_rect.bottom);
		bf.draw_ret_bmp = SelectObject(bf.draw_dc, bf.draw_bmp);
		ReleaseDC(hWnd, hdc);
		bf.draw_brush = CreateSolidBrush(ci.background);
		FillRect(bf.draw_dc, &client_rect, bf.draw_brush);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		// EBhE̔j
		SendMessage(input_wnd, WM_CLOSE, 0, 0);

		SelectObject(bf.draw_dc, bf.draw_ret_bmp);
		DeleteObject(bf.draw_bmp);
		DeleteDC(bf.draw_dc);
		DeleteObject(bf.draw_brush);
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SETFOCUS:
		SetFocus(input_wnd);
		break;

	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);

		BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, ps.rcPaint.bottom,
			bf.draw_dc, ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);

		EndPaint(hWnd, &ps);
		break;

	case WM_LBUTTONDBLCLK:
		break;

	case WM_GAME_OPTION:
		// splatQ[ݒ
		return set_option_splat(hInst, hWnd, &g_si);

	case WM_GAME_INIT:
		// Q[̏
		g_si.game_flag = TRUE;
		for (i = 0; i < g_si.player_cnt; i++) {
			SendMessage(input_wnd, WM_SET_PLAYER_NAME, i, (LPARAM)g_si.pi[i].name);
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, i, g_si.pi[i].score);
		}
		for (; i < PLAYER_MAX; i++) {
			SendMessage(input_wnd, WM_SET_PLAYER_NAME, i, (LPARAM)TEXT(""));
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, i, (LPARAM)TEXT(""));
		}
		g_si.current_round = 1;
		g_si.current_player = 0;
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_si.pi[g_si.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(GetParent(hWnd), WM_SET_BULL_TYPE, g_si.op_bull, 0);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_si.current_player);
		SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, -1);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		SendMessage(hWnd, WM_DRAW_SCORE, g_si.pi[g_si.current_player].score, 0);
		break;

	case WM_GAME_STOP:
		break;

	case WM_CHANGE_FOCUS:
		if (g_si.game_flag == FALSE) {
			break;
		}
		for (i = 0; i < g_si.player_cnt; i++) {
			g_si.pi[i].half = FALSE;
		}
		if (g_si.pi[g_si.current_player].double_in == TRUE && g_si.pi[g_si.current_player].double_in_flag == 0) {
			// _uC
			i = 0;
			for (j = 0; j < THROW_MAX; j++) {
				SendMessage(input_wnd, WM_GET_SCORE_STR, j + 1, (LPARAM)buf);
				if (*buf == CHAR_DOUBLE || double_in_flag == TRUE) {
					i += SendMessage(input_wnd, WM_GET_SCORE, j + 1, 0);
					double_in_flag = TRUE;
					check_score(&g_si, g_si.current_player, g_si.pi[g_si.current_player].score + i);
					if (g_si.pi[g_si.current_player].score + i >= SPLAT_SCORE) {
						break;
					}
				}
			}
		} else {
			// XRA擾
			i = 0;
			for (j = 0; j < THROW_MAX; j++) {
				i += SendMessage(input_wnd, WM_GET_SCORE, j + 1, 0);
				check_score(&g_si, g_si.current_player, g_si.pi[g_si.current_player].score + i);
				if (g_si.pi[g_si.current_player].score + i >= SPLAT_SCORE) {
					break;
				}
			}
		}
		bust_flag = TRUE;
		splat_flag = FALSE;
		if (g_si.pi[g_si.current_player].double_out == TRUE && g_si.pi[g_si.current_player].score + i >= SPLAT_SCORE) {
			// _uAEg
			if (g_si.pi[g_si.current_player].score + i == SPLAT_SCORE) {
				SendMessage(input_wnd, WM_GET_SCORE_STR, j + 1, (LPARAM)buf);
				if (*buf == CHAR_DOUBLE) {
					bust_flag = FALSE;
				} else {
					// half
					i = SPLAT_SCORE - g_si.pi[g_si.current_player].score - (SPLAT_SCORE / 2);
					splat_flag = TRUE;
				}
			} else {
				// BUST
				i = SPLAT_SCORE - g_si.pi[g_si.current_player].score + SPLAT_SCORE - (g_si.pi[g_si.current_player].score + i);
			}
		} else if (g_si.pi[g_si.current_player].score + i > SPLAT_SCORE) {
			// BUST
			i = SPLAT_SCORE - g_si.pi[g_si.current_player].score + SPLAT_SCORE - (g_si.pi[g_si.current_player].score + i);
		} else {
			bust_flag = FALSE;
		}
		if (g_si.pi[g_si.current_player].master_splat == TRUE &&
			g_si.pi[g_si.current_player].score + i == SPLAT_SCORE &&
			g_si.pi[g_si.current_player].master_splat_flag == 0) {
			// half
			i = SPLAT_SCORE - g_si.pi[g_si.current_player].score - (SPLAT_SCORE / 2);
			bust_flag = TRUE;
			splat_flag = TRUE;
		}
		if (g_si.pi[g_si.current_player].score + i < 0) {
			i = 0;
		}
		check_score(&g_si, g_si.current_player, g_si.pi[g_si.current_player].score + i);
		for (j = 0; j < g_si.player_cnt; j++) {
			if (j == g_si.current_player) {
				continue;
			}
			if (g_si.pi[j].half == TRUE) {
				SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, j, (LPARAM)SPLAT_CHAR);
			} else {
				SendMessage(input_wnd, WM_SET_PLAYER_SCORE, j, g_si.pi[j].score);
			}
		}
		if (splat_flag == TRUE) {
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, g_si.current_player, (LPARAM)SPLAT_CHAR);
		} else {
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_si.current_player, g_si.pi[g_si.current_player].score + i);
		}
		SendMessage(hWnd, WM_DRAW_SCORE, g_si.pi[g_si.current_player].score + i, bust_flag);
		if (g_si.pi[g_si.current_player].score + i == SPLAT_SCORE || bust_flag == TRUE) {
			SendMessage(input_wnd, WM_SET_FOCUS_NEXT_BTN, 0, 0);
		}
		break;

	case WM_PREV_PLAYER:
		if (g_si.current_player <= 0 && g_si.current_round <= 1) {
			break;
		}
		SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_si.current_player, g_si.pi[g_si.current_player].score);

		g_si.current_player--;
		if (g_si.current_player < 0) {
			g_si.current_player = g_si.player_cnt - 1;
			g_si.current_round--;
		}
		for (i = 0; i < g_si.player_cnt; i++) {
			g_si.pi[i].score = g_si.pi[g_si.current_player].ri[g_si.current_round - 1].prev_score_list[i];
		}
		if (g_si.pi[g_si.current_player].double_in_flag == g_si.current_round) {
			g_si.pi[g_si.current_player].double_in_flag = 0;
		}
		if (g_si.pi[g_si.current_player].master_splat_flag == g_si.current_round) {
			g_si.pi[g_si.current_player].master_splat_flag = 0;
		}

		// 
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_si.pi[g_si.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_si.current_player);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_SET_SCORE, i + 1, (LPARAM)g_si.pi[g_si.current_player].ri[g_si.current_round - 1].throw_list[i]);
		}
		SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);
		break;

	case WM_NEXT_PLAYER:
		for (i = 0; i < g_si.player_cnt; i++) {
			g_si.pi[i].half = FALSE;
		}
		g_si.pi[g_si.current_player].ri[g_si.current_round - 1].splats = 0;
		if (g_si.pi[g_si.current_player].double_in == TRUE && g_si.pi[g_si.current_player].double_in_flag == 0) {
			// _uC
			for (j = 0; j < THROW_MAX; j++) {
				SendMessage(input_wnd, WM_GET_SCORE_FULL_STR, j + 1, (LPARAM)g_si.pi[g_si.current_player].ri[g_si.current_round - 1].throw_list[j]);
				SendMessage(input_wnd, WM_GET_SCORE_STR, j + 1, (LPARAM)buf);
				if (*buf == CHAR_DOUBLE || g_si.pi[g_si.current_player].double_in_flag != 0) {
					g_si.pi[g_si.current_player].score += SendMessage(input_wnd, WM_GET_SCORE, j + 1, 0);
					if (g_si.pi[g_si.current_player].double_in_flag == 0) {
						g_si.pi[g_si.current_player].double_in_flag = g_si.current_round;
					}
					master_splat_flag = set_score(&g_si, g_si.current_player);
					if (g_si.pi[g_si.current_player].master_splat_flag == 0 && master_splat_flag == TRUE) {
						g_si.pi[g_si.current_player].master_splat_flag = g_si.current_round;
					}
					if (g_si.pi[g_si.current_player].score >= SPLAT_SCORE) {
						break;
					}
				}
			}
		} else {
			// XRA擾
			for (j = 0; j < THROW_MAX; j++) {
				SendMessage(input_wnd, WM_GET_SCORE_FULL_STR, j + 1, (LPARAM)g_si.pi[g_si.current_player].ri[g_si.current_round - 1].throw_list[j]);
				// XRA擾
				g_si.pi[g_si.current_player].score += SendMessage(input_wnd, WM_GET_SCORE, j + 1, 0);
				master_splat_flag = set_score(&g_si, g_si.current_player);
				if (g_si.pi[g_si.current_player].master_splat_flag == 0 && master_splat_flag == TRUE) {
					g_si.pi[g_si.current_player].master_splat_flag = g_si.current_round;
				}
				if (g_si.pi[g_si.current_player].score >= SPLAT_SCORE) {
					break;
				}
			}
		}
		if (g_si.pi[g_si.current_player].double_out == TRUE && g_si.pi[g_si.current_player].score >= SPLAT_SCORE) {
			// _uAEg
			if (g_si.pi[g_si.current_player].score == SPLAT_SCORE) {
				SendMessage(input_wnd, WM_GET_SCORE_STR, j + 1, (LPARAM)buf);
				if (*buf != CHAR_DOUBLE) {
					// half
					g_si.pi[g_si.current_player].score = (g_si.pi[g_si.current_player].score + 1) / 2;
					if (g_si.pi[g_si.current_player].master_splat_flag == 0) {
						g_si.pi[g_si.current_player].master_splat_flag = g_si.current_round;
					}
				}
			} else {
				// BUST
				g_si.pi[g_si.current_player].score = SPLAT_SCORE + SPLAT_SCORE - g_si.pi[g_si.current_player].score;
			}
		} else if (g_si.pi[g_si.current_player].score > SPLAT_SCORE) {
			// BUST
			g_si.pi[g_si.current_player].score = SPLAT_SCORE + SPLAT_SCORE - g_si.pi[g_si.current_player].score;
		}
		if (g_si.pi[g_si.current_player].master_splat == TRUE &&
			g_si.pi[g_si.current_player].score == SPLAT_SCORE &&
			g_si.pi[g_si.current_player].master_splat_flag == 0) {
			// half
			g_si.pi[g_si.current_player].score = (g_si.pi[g_si.current_player].score + 1) / 2;
			g_si.pi[g_si.current_player].master_splat_flag = g_si.current_round;
		}

		// XRAݒ
		master_splat_flag = set_score(&g_si, g_si.current_player);
		if (g_si.pi[g_si.current_player].master_splat_flag == 0 && master_splat_flag == TRUE) {
			g_si.pi[g_si.current_player].master_splat_flag = g_si.current_round;
		}
		for (i = 0; i < g_si.player_cnt; i++) {
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, i, g_si.pi[i].score);
		}
		g_si.pi[g_si.current_player].ri[g_si.current_round - 1].score_list = g_si.pi[g_si.current_player].score;

		if (g_si.pi[g_si.current_player].score == SPLAT_SCORE) {
			// Q[NA
			g_si.game_flag = FALSE;
			SendMessage(GetParent(hWnd), WM_GAME_END, 0, 0);
			SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, g_si.current_player);
			wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), g_si.pi[g_si.current_player].name);
			SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
			MessageBox(hWnd, buf, message_get_res(IDS_STRING_TITLE_SPLAT), MB_ICONINFORMATION);
			break;
		}
		// vC[`FW
		g_si.current_player++;
		if (g_si.current_player >= g_si.player_cnt) {
			g_si.current_player = 0;
			g_si.current_round++;
		}
		if (g_si.current_round > g_si.round_cnt) {
			// SEhI
			g_si.game_flag = FALSE;
			SendMessage(GetParent(hWnd), WM_GAME_END, 0, 0);
			for (j = 0, i = 1; i < g_si.player_cnt; i++) {
				if (g_si.pi[j].score < g_si.pi[i].score) {
					j = i;
				}
			}
			for (i = 0; i < g_si.player_cnt; i++) {
				if (j != i && g_si.pi[j].score == g_si.pi[i].score) {
					break;
				}
			}
			if (g_si.player_cnt > 1 && g_si.player_cnt > i) {
				// 
				lstrcpy(buf, message_get_res(IDS_STRING_TIE));
			} else {
				SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, j);
				wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), g_si.pi[j].name);
			}
			SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
			MessageBox(hWnd, buf, message_get_res(IDS_STRING_TITLE_SPLAT), MB_ICONINFORMATION);
			break;
		}

		// ߂pɑޔ
		for (i = 0; i < g_si.player_cnt; i++) {
			g_si.pi[g_si.current_player].ri[g_si.current_round - 1].prev_score_list[i] = g_si.pi[i].score;
		}

		// 
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_si.pi[g_si.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_si.current_player);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_SET_SCORE, i + 1, (LPARAM)g_si.pi[g_si.current_player].ri[g_si.current_round - 1].throw_list[i]);
		}
		SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);
		break;

	case WM_DRAW_SCORE:
		GetClientRect(hWnd, &client_rect);
		GetClientRect(input_wnd, &input_rect);
		client_rect.bottom -= input_rect.bottom;
		score_draw(&bf, &client_rect, &g_si, wParam, lParam);
		InvalidateRect(hWnd, &client_rect, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_SHOW_SCORE:
		if (wParam != -1) {
			g_si.list_index = wParam;
		} else {
			g_si.list_index = (g_si.game_flag == FALSE) ? 0 : g_si.current_player;
		}
		show_score_splat_list(hInst, hWnd, &g_si);
		break;

	case WM_SAVE_SCORE:
		score_save(hWnd, &g_si);
		break;

	case WM_SET_SCORE:
	case WM_SET_INPUT_ENABLE:
		return SendMessage(input_wnd, msg, wParam, lParam);

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_ACCEL_TAB:
		case ID_ACCEL_STAB:
		case ID_ACCEL_RETURN:
		case IDC_BUTTON_NEXT:
			SendMessage(input_wnd, WM_COMMAND, wParam, lParam);
			break;
		}
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * score_splat_regist - EBhENX̓o^
 */
BOOL score_splat_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = (WNDPROC)score_splat_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * score_splat_create - EBhE̍쐬
 */
HWND score_splat_create(const HINSTANCE hInstance, const HWND pWnd, int id, const int height)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindow(WINDOW_CLASS,
		TEXT(""),
		WS_CHILD | WS_TABSTOP,
		0, 0, SCORE_WIDTH, height, pWnd, (HMENU)id, hInstance, NULL);
	return hWnd;
}
/* End of source */
